#!/usr/bin/python
# BSD License
# Copyright (c) 2009, Peter Banka et al
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
# * Neither the name of the GE Security nor the names of its contributors may
#   be used to endorse or promote products derived from this software without
#   specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

'''A [PinshCmd] command which allows a user to enter free text as an argument'''


import PinshCmd
from bombardier_core.static_data import PARTIAL

class Expression(PinshCmd.PinshCmd):
    def __init__(self, name = "expression"):
        PinshCmd.PinshCmd.__init__(self, name)
        self.help_text = name+"\tan arbitrary expression"
        self.level = 99
        self.cmd_owner = 0

    def match(self, tokens, index):
        #print "exp-match %s\n" % ( len(tokens) - index )
        return PARTIAL, len(tokens) - index

    def preferred_names(self, tokens, index):
        #print tokens[-1],'\n'
        return [tokens[-1], tokens[-1]]



#!/usr/bin/python
# BSD License
# Copyright (c) 2009, Peter Banka et al
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
# * Neither the name of the GE Security nor the names of its contributors may
#   be used to endorse or promote products derived from this software without
#   specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

'''A special [PinshCmd] command which is the root of all other commands. Slash
Does not do anything itself, but merely has other PinshCmd objects which are
its children'''

import traceback, StringIO
import exceptions, readline
import sys
import PinshCmd, libUi
from Exceptions import AmbiguousCommand, UnknownCommand, ServerException
from Exceptions import CommandError, ServerTracebackException
from Exceptions import MachineStatusException
from SystemStateSingleton import SystemState
system_state = SystemState()
from bombardier_core.static_data import FAIL, OK

class Slash(PinshCmd.PinshCmd):
    '''Root-level PinshCmd object, has no name.'''
    def __init__(self, children):
        PinshCmd.PinshCmd.__init__(self, "")
        self.children = children
        self.cmd_owner = 1
        self.help_text = ''

    def complete_tokens(self, tokens):
        completed_tokens = []
        current_tokens = []
        for token in tokens:
            current_tokens.append(token)
            names, token_delimeter = self.get_names_and_token_delimeter(current_tokens)
            if not names:
                raise UnknownCommand(' '.join(current_tokens))
            if token in names:
                completed_tokens.append(token)
            else:
                if len(names) > 1:
                    raise AmbiguousCommand(current_tokens[-1], names)
                completed_tokens.append(names[0])
        return completed_tokens

    def run(self, tokens, no_flag):
        'finds the correct object and runs a command'
        if tokens[-1] == '':
            tokens = tokens[:-1]
        owner = self.find_last_responsible_child(tokens, 0)
        if not owner:
            return FAIL, []
        try:
            tokens = self.complete_tokens(tokens)
            return_value = owner.cmd(tokens, no_flag)
        except AmbiguousCommand, amb_err:
            return FAIL, [str(amb_err)]
        except UnknownCommand, unk_err:
            return FAIL, [' ', str(unk_err)]
        except ServerTracebackException, trc:
            return FAIL, ['Server traceback: ', [trc.traceback]]
        if return_value == None or len(return_value) != 2:
            return OK, []
        else:
            status = return_value[0]
            output = return_value[1]
            #if owner.log_command:
                #cmd = log(no_flag, tokens, status, output)
                #system_state.comment_commands.append(cmd)
            system_state.globals["output"] = output
            system_state.globals["status"] = status
            return status, output

    def process_command(self, command):
        'When somebody hits return in the shell, this method handles it.'
        try:
            no_flag, help_flag, tokens, comment = libUi.process_input(command)
            if help_flag: # Process the [?] key first
                self.find_help(tokens, 0)
                return None
            if len(tokens) == 0:
                # somebody just pressed return for no reason
                return OK, []
            tokens = system_state.get_state_tokens(tokens)
            status, output = self.run(tokens, no_flag)
            #if comment:
                #makeComment(comment) # MISSING
            libUi.user_output(output, status)
            return status, output
        except ServerException, err:
            output = ["ERROR: Cannot communicate with CNM Server"]
            output.append(str(err))
            libUi.user_output(output, FAIL)
        except MachineStatusException, err:
            output = ["ERROR: Machine status is incomplete"]
            output.append(str(err))
            libUi.user_output(output, FAIL)
        except exceptions.SystemExit:
            #if system_state.comment_commands:
                #makeComment()
            sys.exit(0)
        except CommandError, err:
            system_state.fp_out.write("\n %% %s\n\n" % err )
        except Exception, err:
            msg = " %%%% Error detected in %s (%s)." % (command, err)
            system_state.fp_err.write( msg )
            tb_str = StringIO.StringIO()
            traceback.print_exc(file=tb_str)
            tb_str.seek(0)
            data = tb_str.read()
            ermsg = ''
            for line in data.split('\n'):
                ermsg += "\n||>>>%s" % line
            system_state.fp_err.write(ermsg)
            system_state.fp_err.write("\n")
        return FAIL, ["process_command Excepted"]

    @classmethod
    def get_names(cls, objects, tokens, index):
        """
        There is a tree of objects, such as:
         slash:
            - Show:
                - machine
                    - <machine_name>
                - package
                    - <package_name>
            - Set
                - machine
                    - <machine_name>
                - package
                    - <package_name>
            - Terminal
                - width
                - color
            - etc.
        ...and there is a set of user input, such as ['s']
        It is the job of this function to return a list of name objects that can
        be used to provide command-line completion.
        objects -- a list of PinshCmd objects
        tokens -- a list of string literals that represent strings the user
                  typed in
        index -- the current position of evaluation
        """
        names = []
        for obj in objects:
            new_name = obj.preferred_names(tokens, index)
            if type(new_name) == type("string"):
                names.append(new_name)
            else:
                names = names + new_name
        return names

    def get_names_and_token_delimeter(self, tokens):
        index = 0
        if tokens == []:
            completion_objects = self.children
        else:
            completion_objects, index = self.find_completions(tokens, 0)
        if len(completion_objects) == 0:
            return None, ' '
        token_delimeter = completion_objects[0].token_delimeter
        names = self.get_names(completion_objects, tokens, index-1)
        return names, token_delimeter

    def complete(self, _text, status):
        '''Command line completer, called with [tab]
        or [?] (if we could bind it)
        _text -- text that readline sends as what the user enters
        status -- the index into the completion list'''
        try:
            if status > 0:
                if status >= len(self.names):
                    return None
                return self.names[status]
            else:
                _no_flag, _help_flag, tokens, _comment = \
                    libUi.process_input(readline.get_line_buffer())
                tokens = system_state.get_state_tokens(tokens)
                # this is where we would process help if
                # we could bind the '?' key properly
                names, token_delimeter = self.get_names_and_token_delimeter(tokens)
                self.names = names
                if not self.names:
                    return []
                if len(self.names) == 1:
                    return self.names[0] + token_delimeter
                if self.names:
                    return self.names[0]
                return []
        except StandardError, err:
            sys.stderr.write(" %%%% Error detected in %s (%s)." % (file, err))
            tb_str = StringIO.StringIO()
            traceback.print_exc(file=tb_str)
            tb_str.seek(0)
            data = tb_str.read()
            ermsg = ''
            for line in data.split('\n'):
                ermsg += "\n||>>>%s" % line
            sys.stderr.write(ermsg)
            sys.stderr.write(" %%%% Error ocurred in %s" % file)
            print
            return []




#!/usr/bin/env python

# BSD License
# Copyright (c) 2009, Peter Banka et al
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
# * Neither the name of the GE Security nor the names of its contributors may
#   be used to endorse or promote products derived from this software without
#   specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

'''A [PinshCmd] that is used for bomsh completion testing'''

import PinshCmd, Integer
import MultipleChoice
from bombardier_core.static_data import OK, FAIL
from SystemStateSingleton import SystemState
system_state = SystemState()
from Exceptions import AmbiguousCommand, UnknownCommand

class CliTest(PinshCmd.PinshCmd):
    'Used in unit testing only'
    def __init__(self):
        PinshCmd.PinshCmd.__init__(self, "cli")
        self.help_text = "cli\tdo not use this"
        self.spammy = PinshCmd.PinshCmd("spammy", "spammy\tTHIS IS A TEST")
        self.spammy2  = PinshCmd.PinshCmd("spammy2", "spammy2\tTHIS IS A TEST")
        self.spotty  = PinshCmd.PinshCmd("spotty", "spotty\tTHIS IS A TEST")
        help_text = ["AA", "BB", "CC"]
        self.options = MultipleChoice.MultipleChoice(choices = help_text,
                                                     help_text = help_text)
        self.children = [self.spammy, self.spammy2, self.spotty]
        self.num = Integer.Integer(min_value = 0, max_value = 1000, name = "<int>")
        self.spammy.children = [self.num]
        self.spammy2.children = [self.num]
        self.spotty.children = [self.options]
        self.cmd_owner = 1

    def cmd(self, tokens, no_flag):
        'Used in unit testing only'
        if no_flag:
            return OK, ["YOU SAID NO"]
        if len(tokens) < 3:
            return FAIL, ["YOU NEED TO TYPE IN SOME MORE STUFF."]

        command = self.find_one_child(tokens, 1)
        return [OK, [tokens[0], command, tokens[2]]]






#!/usr/bin/python
# BSD License
# Copyright (c) 2009, Peter Banka et al
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
# * Neither the name of the GE Security nor the names of its contributors may
#   be used to endorse or promote products derived from this software without
#   specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

'''Provides the base class for all command-line token object fields
or commands'''

from bombardier_core.static_data import PARTIAL, COMPLETE, INCOMPLETE
from bombardier_core.static_data import NO_MATCH
from Exceptions import AmbiguousCommand, UnknownCommand
from SystemStateSingleton import SystemState, ENABLE
system_state = SystemState()

DEBUGGING = False

# find the names of all the objects given to me
def convert_tokens_to_string(tokens, delimeter=' '):
    '''Used in error reporting.
    delimeter -- typically a space, but some tokens are separated
                 using other characters'''
    retVal = ''
    for token in tokens:
        retVal += token+delimeter
    return retVal[:-1]

class PinshCmd:
    '''Base class for all command and field objects. Really should be
    called CommandToken, because it represents a single executable
    or nameable token in an input string.'''

    def __init__(self, name, help_text = "<cr>",
                 auth = ENABLE, token_delimeter = ' '):
        '''
        name -- name of the command or field
        help_text -- explanatory information about this command
        auth -- authorization required to run this command
        token_delimeter -- character that separates this command or field
                           from other ones.
        '''
        self.my_name = name
        self.help_text = help_text
        self.children = []
        self.cmd_owner = 0
        self.auth = auth
        self.token_delimeter = token_delimeter
        self.names = []
        self.log_command = False

    def __repr__(self):
        return self.my_name

    def find_one_child(self, tokens, index):
        '''Sometimes it's handy to be able to locate one of your
        children, based on a token name and index.'''
        possible_choices = []
        child_names = [child.my_name for child in self.children]
        for name in child_names:
            if name.find(tokens[index]) == 0:
                possible_choices.append(name)

        if not possible_choices:
            raise UnknownCommand(tokens[index])
        if len(possible_choices) > 1:
            if tokens[index] in child_names:
                possible_choices = [tokens[index]]
            else:
                raise AmbiguousCommand(tokens[index], possible_choices)
        return possible_choices[0]

    def preferred_names(self, _tokens, _index):
        '''How is this command or field represented in the shell? Often
        it is represented as a fixed text item, such as 'show' or 'set',
        but often it is a name of a host, such as 'localhost' or 'server10'.
        If a class wants to provide a customized name, it must override this
        method.'''
        return [self.my_name]

    def acceptable_names(self, tokens, index):
        '''Because commands can be shortened, it is possible that "sh" could
        be used in place of "show", for example. preferred_names(), on the
        other hand represents the name that best matches this object.
        '''
        return self.preferred_names(tokens, index)

    def match(self, tokens, index):
        '''Used to determine if what the user has typed in could potentially
        be handled by this object. For example, if a user typed in 'sh', and
        this object is called 'show', then this object would return PARTIAL. If
        the user typed in 'show', it would return COMPLETE, and if the user
        typed in "showme" this method would return NO_MATCH.'''
        if self.auth > system_state.auth:
            self.dbg("NOT Authorized")
            return NO_MATCH, 1
        self.dbg("tokens[index] = %s" % tokens[index])
        if tokens[index] == '':
            return PARTIAL, 1
        if self.my_name == tokens[index]:
            return COMPLETE, 1
        if self.my_name.startswith(tokens[index]):
            return PARTIAL, 1
        else:
            return NO_MATCH, 1

    @classmethod
    def print_error_msg(cls, tokens, index, message = "Unrecognized Command"):
        'Prints a customized error message, showing the position of the error'
        preamble = ' '*len(system_state.get_prompt())
        ok_tokens = convert_tokens_to_string(tokens[:index])
        print "%s%s" % (preamble, convert_tokens_to_string(tokens))
        print "%s%s ^" % (preamble, " "*len(ok_tokens))
        print " %% %s" % message

    def find_possible_completions(self, tokens, index):
        '''Look through all the children of this object, and based
        on the tokens/index, find one or more children that could
        possibly match what the user is looking for'''

        completion_objects = []
        incomplete_objects = []
        match_len = 0
        for child in self.children:
            if child.auth > system_state.auth:
                continue
            match_value, length = child.match(tokens, index)
            d_tmp = "find_completions match_value: %s length: %d"
            if match_value == INCOMPLETE:
                self.dbg(d_tmp % (match_value, length))
                incomplete_objects.append(child)
            if match_value == PARTIAL:
                self.dbg(d_tmp % (match_value, length))
                if length > match_len:
                    match_len = length
                completion_objects.append(child)
            elif match_value == COMPLETE: # go see if there are more tokens!
                self.dbg(d_tmp % (match_value, length))
                new_name = child.preferred_names(tokens, index)[0]
                tokens[index+length-1] = new_name.split(' ')[-1]
                self.dbg("NEW TOKEN: %s" % tokens[index])
                completion_objects = [child]
                match_len = length
                break
        return completion_objects, incomplete_objects, match_len

    def find_completions(self, tokens, index):
        '''This is called by complete. When complete is calling this it wants
        a list of objects that could be completions for the final token.'''
        return_error = 1
        d_tmp = "find_completions: self.my_name: %s tokens: %s (%d)"
        self.dbg(d_tmp % (self.my_name, tokens, len(tokens)))
        if len(tokens[index:]) == 0: # no tokens left, I must be who you want!
            self.dbg("find_completions: FOUND at TOP")
            return [self], index
        if tokens[index] == '':
            if len(self.children) > 0:
                output = []
                for child in self.children:
                    if child.auth <= system_state.auth:
                        output.append(child)
                return output, index+1
            return_error = 0

        completion_objects, incomplete_objects, match_len = \
            self.find_possible_completions(tokens, index)

        if len(completion_objects) == 1:
            new_child = completion_objects[0]
            if index+match_len >= len(tokens):
                return [new_child], index+1
            return new_child.find_completions(tokens, index+match_len)
        elif len(completion_objects) == 0: # No matches: go away.
            if len(incomplete_objects) > 0:
                print
                msg = "Command cannot be completed."
                self.print_error_msg(tokens, index, msg)
                system_state.reprompt()
                return [], 1
            if return_error:
                print
                self.print_error_msg(tokens, index)
                system_state.reprompt()
            return [], index
        else: # we have a few possible matches, return them all
            return completion_objects, index

    @classmethod
    def dbg(cls, msg):
        'Little debugger'
        if DEBUGGING:
            print "******", msg

    def find_help(self, tokens, index):
        '''When complete is calling this, it wants help for all the
        possible arguments of the last token, which should be unambiguous.
        No return value is necessary'''
        self.dbg("find_help: %s tokens: %s" % (self.my_name, tokens))
        # no tokens left, I must be who you want!
        if len(tokens[index:]) == 0 or tokens[index] == '':
            self.dbg("my_name: %s" % self.my_name)
            self.help()
            return
        match_len = 0
        completion_objects = []
        for child in self.children:
            match_value, length = child.match(tokens, index)
            self.dbg("child: %s / %s / %s" % (child, match_value, length))
            if match_value == PARTIAL:
                if length > match_len:
                    match_len = length
                completion_objects.append(child)
            elif match_value == COMPLETE:
                completion_objects = [child]
                match_len = length
                break
        if len(completion_objects) == 1:
            return completion_objects[0].find_help(tokens, index + match_len)
        elif len(completion_objects) == 0:
            self.print_error_msg(tokens, index)
            return
        else:
            self.print_error_msg(tokens, index, "Ambiguous command")
            return

    def find_last_responsible_child(self, tokens, index):
        '''Run is calling this method to find the last object in the chain
        that is willing to run cmd() on the set of tokens on the list.
        Must be unambiguous. Assume coming into the routine that the
        current object ("self") is a cmd_owner.'''
        d_tmp = "find_last_responsible_child: %s %s index: %d"
        self.dbg( d_tmp % (self.my_name, tokens, index) )
        if len(tokens[index:]) == 0 or tokens[index] == '':
            # complete and no more tokens. Object takes responsibility
            return self
        owners = []
        arguments = []
        match_len = 0
        for child in self.children:
            match_value, length = child.match(tokens, index)
            self.dbg("Match: %s / %s " % ( child.my_name, match_value ))
            if match_value == PARTIAL:
                if length > match_len:
                    match_len = length
                if child.cmd_owner:
                    owners.append(child)
                else:
                    arguments.append(child)
            elif match_value == COMPLETE:
                new_name = child.acceptable_names(tokens, index)[0]
                tokens[index+length-1] = new_name
                self.dbg("NEW TOKEN: %s" % tokens[index])
                if child.cmd_owner:
                    return child.find_last_responsible_child(tokens,
                                                             index + length)
                else:
                    return self
        if len(owners) == 1:
            owner = owners[0]
            completed_name = owner.acceptable_names(tokens, index)[0]
            tokens[index+length-1] = completed_name
            self.dbg("NEW TOKEN: %s" % tokens[index])
            return owner.find_last_responsible_child(tokens, index+match_len)
        elif len(owners) == 0:
            if len(arguments) > 0:
                return self
            self.print_error_msg(tokens, index)
            return None
        else: # more than one owner-- need to be unambiguous
            self.print_error_msg(tokens, index, "Ambiguous command")
            return None

    def help(self):
        'pretty-print the help strings of all my children'
        help_text = []
        max_len = 0
        if len(self.children) == 0:
            print "<cr>\n"
            return

        for child in self.children:
            if system_state.auth < child.auth:
                continue
            if child.help_text.rfind('\n') != -1:
                for help_line in child.help_text.split('\n'):
                    cmd, doc = help_line.split('\t')
                    help_text.append([cmd, doc])
                    if len(cmd) > max_len:
                        max_len = len(cmd)
            else:
                cmd, doc = child.help_text.split('\t')
                help_text.append([cmd, doc])
            if len(cmd) > max_len:
                max_len = len(cmd)

        help_text.sort()

        for help_line in help_text:
            cmd = help_line[0]
            text = help_line[1]
            spaces = max_len - len(cmd) + 5
            print "  ", cmd + ' ' * spaces + text



#!/usr/bin/python
# BSD License
# Copyright (c) 2009, Peter Banka et al
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
# * Neither the name of the GE Security nor the names of its contributors may
#   be used to endorse or promote products derived from this software without
#   specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

'''A [PinshCmd] command which allows a user to configure the system'''


import sys, os, time, glob, md5, random
import PinshCmd, libUi, Expression
from bombardier_core.static_data import OK, FAIL
from bombardier_core.static_data import LOG_LEVEL_LOOKUP
from MultipleChoice import MultipleChoice
#from bombardier_server.cli.SystemStateSingleton import SystemState
from SystemStateSingleton import SystemState
from Exceptions import CommandError, UnexpectedDataException
system_state = SystemState()

class Set(PinshCmd.PinshCmd):
#    '''bomsh# set configuration-key abc123
#       [OK, ['Configuration key set.']]
#       bomsh# set configuration-key foobar
#       [FAIL, ['Invalid configuration key.']]
    '''
       bomsh# set log-level debug
       [OK, ['Logging output set to debug']]
       bomsh# set log-level info
       [OK, ['Logging output set to info']]
    '''
    def __init__(self):
        PinshCmd.PinshCmd.__init__(self, "set", "set\tset a configuration value")
        # TOP LEVEL
        help = "configuration-key\tset the key for encrypting configuation items"
        home_help = "server-home\tset the root directory for configuration"
        self.configuration_key = PinshCmd.PinshCmd("configuration-key", help)
        server_home = PinshCmd.PinshCmd("server-home", home_help)
        server_home.children = [Expression.Expression("path")]
        self.configuration_key.children = [Expression.Expression("password")]

        log_level = PinshCmd.PinshCmd("log-level", "set degree of logging to display to terminal")
        choices = ["debug", "info", "warning", "error", "critical"]
        choice_help = ["Maximum debugging", "Normal amounts of logs", "Reduced logging",
                       "Much reduced logging", "Minimum logging"]
        choice_field = MultipleChoice(choices=choices, help_text=choice_help)
        log_level.children = [choice_field]
        self.children = [self.configuration_key, server_home, log_level]
        self.level = 0
        self.cmd_owner = 1

    def cmd(self, tokens, no_flag):
        if tokens[1] == "configuration-key":
            if len(tokens) == 2:
                configuration_key = libUi.pwd_input("Enter CI decryption password: ")
            else:
                configuration_key = tokens[2]
            url = "/json/dispatcher/set-password"
            post_data = {"password": configuration_key}
            syr = system_state.cnm_connector.service_yaml_request
            output = syr(url, post_data=post_data)
            return output["command_status"], output["command_output"]

        elif tokens[1] == "server-home":
            if len(tokens) != 3:
                raise CommandError("Incomplete command.")
            connector = system_state.cnm_connector
            server_home = tokens[2]
            try:
                connector.sync_server_home(server_home)
            except UnexpectedDataException, ude:
                return FAIL, [str(ude)] 
            return OK, ["Server home set to %s" % server_home]

        elif tokens[1] == "log-level":
            new_log_level = LOG_LEVEL_LOOKUP[tokens[2].upper()]
            system_state.log_level = new_log_level
            return [OK, ["Logging output set to %s" % tokens[2]]]



# BSD License
# Copyright (c) 2009, Peter Banka et al
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
# * Neither the name of the GE Security nor the names of its contributors may
#   be used to endorse or promote products derived from this software without
#   specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

'''A [PinshCmd] field which provides for selecting from a group'''

import PinshCmd
from bombardier_core.static_data import NO_MATCH, COMPLETE, PARTIAL

class MultipleChoice(PinshCmd.PinshCmd):
    '''A Field which allows the user a simple way of choosing from a group'''
    def __init__(self, choices = None, help_text = None):
        '''
        choices -- a list of possible choices
        help_text -- a list of what each choice means
        '''
        PinshCmd.PinshCmd.__init__(self, "multiple")
        self.help_text = ""
        self.choices = choices
        if not choices:
            self.help_text = "error\t--This module needs attention--"
            return
        if len(choices) != len(help_text):
            self.help_text = "error\t--This module needs attention--"
            return
        for index in range(0, len(choices)):
            self.help_text += choices[index]+'\t'+help_text[index]+'\n'
        self.help_text = self.help_text[:-1]
        self.cmd_owner = 0

    def match(self, tokens, index):
        '''determines if the user typed in an option that the system
        recognizes'''
        possible_choices = self.preferred_names(tokens, index)
        if len(possible_choices) == 0:
            return NO_MATCH, 1
        if len(possible_choices) == 1:
            return COMPLETE, 1
        return PARTIAL, 1

    def preferred_names(self, tokens, index):
        '''provides the full name of the choice if a partial one
        was given.'''
        possible_choices = []
        if tokens[index] == '':
            return self.choices
        for choice in self.choices:
            if choice.find(tokens[index]) == 0:
                possible_choices.append(choice)
        return possible_choices



#!/usr/bin/python
# BSD License
# Copyright (c) 2009, Peter Banka et al
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
# * Neither the name of the GE Security nor the names of its contributors may
#   be used to endorse or promote products derived from this software without
#   specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

'Classes that hold command-line interface-specific exceptions'

class ExceptionBase(Exception):
    def __init__(self):
        Exception.__init__(self)
    def __str__(self):
        return self.__repr__()

class CommandError(Exception):
    '''If any command is improper, this can be thrown instead of a fail'''
    def __init__(self, error_string):
        Exception.__init__(self)
        self.error_string = error_string
    def __repr__(self):
        return [self.error_string]
    def __list__(self):
        return self.__repr__()
    def __str__(self):
        return '\n'.join(self.__repr__())

class HostNotEnabledException(Exception):
    '''It is necessary to enable a host before trying to work with it.
    if a user attempts to do something with a host that is not enabled,
    this exception throws.'''
    def __init__(self, server):
        Exception.__init__(self)
        self.server = server
    def __repr__(self):
        return "Host %s is not enabled for this user" % self.server

class ConfigFileException(Exception):
    '''Your config file (~/.bomsh_config) stores critical setup info.
    If there is a problem reading or writing to it, this throws.'''
    def __init__(self, message, fileName):
        Exception.__init__(self)
        self.fileName = fileName
        self.message = message
    def __repr__(self):
        return "%% Error processing config file %s: %s" \
               % (self.fileName, self.message)
    def __str__(self):
        return self.__repr__()

class ServerTracebackException(ExceptionBase):
    def __init__(self, traceback_lines):
        ExceptionBase.__init__(self)
        self.traceback = traceback_lines
    def __repr__(self):
        return "Server Exception"

class MachineUnavailableException(ExceptionBase):
    def __init__(self, msg):
        self.msg = msg
    def __repr__(self):
        return "Machine is not available: %s" % self.msg

class MachineStatusException(ExceptionBase):
    def __init__(self, msg):
        self.msg = msg
    def __repr__(self):
        return "Error in status data: %s" % self.msg

class ServerException(Exception):
    '''It is necessary to communicate with the RESTful Bombardier web server
    to make the CLI work. If there is an issue with a command to that server
    then this throws.'''
    def __init__(self, url, curl_err, http_code):
        Exception.__init__(self)
        self.url = url
        self.curl_err = curl_err
        self.http_code = http_code
    def __repr__(self):
        return "Can't connect to %s (%s) (%d)" % (self.url, self.curl_err, self.http_code)
    def __str__(self):
        return self.__repr__()

class UnexpectedDataException(Exception):
    '''Whenever we are receiving information from a server that we're not
    expecting.'''
    def __init__(self, reason):
        Exception.__init__(self)
        self.reason = reason
    def __repr__(self):
        return "Got unexpected data from the server (%s)" % self.reason
    def __str__(self):
        return self.__repr__()

class UnknownCommand(Exception):
    "Someone entered a command that doesn't make sense"
    def __init__(self, command):
        Exception.__init__(self)
        self.command = command
    def __repr__(self):
        return "Unknown command (%s)" % self.command
    def __str__(self):
        return self.__repr__()

class AmbiguousCommand(Exception):
    "Someone entered a command that could have matched more than one option"
    def __init__(self, command, options):
        Exception.__init__(self)
        self.command = command
        self.options = options
    def __repr__(self):
        return 'Command "%s" ambiguous -- could be %s.' % (self.command, self.options)
    def __str__(self):
        return self.__repr__()

class MachineTraceback(Exception):
    "When trying to execute a command on a machine, it raised."
    def __init__(self, url, traceback):
        Exception.__init__(self)
        self.url = url
        self.traceback = traceback
    def __repr__(self):
        return 'Raised an exception in server url: %s' % (self.url)
    def __str__(self):
        return self.__repr__()




from SystemStateSingleton import SystemState
from bombardier_core.static_data import OK, FAIL, STRING_TO_RETURN_VALUE_MAPPING
import glob, sys, StringIO, yaml
import libUi
import re
import Slash
IGNORE_MODULE_NAMES = ['tester', 'libUi', 'Slash', 'SystemStateSingleton',
                       '_version']

def get_cls(module_name, module_to_test):
    setup_test = None
    if module_name in IGNORE_MODULE_NAMES:
        return None, None
    if modules_to_test:
        if not module_name in modules_to_test:
            return None, None
    module = __import__(module_name)
    if not hasattr(module, module_name):
        return None, None
    if hasattr(module, "setup_test"):
        setup_test = module.setup_test
    cls = getattr(module, module_name)
    return cls, setup_test

class TestCmd:
    def __init__(self, command, status, output_lines):
        self.command = command
        self.status = status
        self.output_lines = output_lines

def get_tests(cls, module_name):
    tests = []
    if module_name in IGNORE_MODULE_NAMES:
        return tests
    module = __import__(module_name)
    if not hasattr(module, module_name):
        return tests
    cls = getattr(module, module_name)
    doc_str = cls.__doc__
    if not doc_str:
        return tests
    command = ''
    for line in doc_str.split('\n'):
        if command:
            #result = re.compile("(\[(OK|FAIL), \[.*\]\])").findall(line)
            result = re.compile("(\[(OK|FAIL), .*\])").findall(line)
            if result:
                yaml_str = result[0][0]
                try:
                    parsed_test = yaml.load(yaml_str)
                    status = STRING_TO_RETURN_VALUE_MAPPING[parsed_test[0]]
                    output_lines = parsed_test[1]
                    tests.append(TestCmd(command, status, output_lines))
                except yaml.parser.ParserError:
                    print "Invalid result: ", line
                    break
            command = ''
        else:
            result = re.compile('bomsh\# (.*)').findall(line)
            if result:
                command = result[0]
    return tests

def compare_expected(expected, received):
    exp_list = [ x for x in expected.split(' ') if x != '' ]
    rec_list = [ x for x in received.split(' ') if x != '' ]
    if len(exp_list) != len(rec_list):
        return False
    for i in xrange(len(exp_list)):
        exp = exp_list[i]
        rec = rec_list[i]
        if exp == rec:
            continue
        elif exp == "==UNKNOWN==":
            continue
        else:
            return False
    return True

def run_tests(cls, tests, debug):
    object = cls()
    slash = Slash.Slash([object])

    output = {}
    for test_cmd in tests:
        command = test_cmd.command
        expected_status = test_cmd.status
        expected_cmd_output = test_cmd.output_lines
        if debug:
            print "TESTING: ", cls, command
            print "Expected result:", expected_status, expected_cmd_output
        #no_flag, help_flag, tokens, comment = libUi.process_input(command)
        cmd_status, cmd_output = slash.process_command(command.strip())
        if debug:
            print "COMMAND OUTPUT:",cmd_output
        if cmd_status != expected_status:
            print "STATUS FAIL: (%s) != (%s)" % (cmd_status, expected_status)
            output[command] = "FAIL"
            continue
        if len(cmd_output) != len(expected_cmd_output):
            output[command] = "FAIL"
            print "LENGTH FAIL: len(%s) != len(%s)" % (cmd_output, expected_cmd_output)
            continue
        if type(cmd_output) == type(['list']):
            for index in range(0,len(cmd_output)):
                received_user_line = cmd_output[index].strip()
                expected_user_line = expected_cmd_output[index].strip()
                if not compare_expected(expected_user_line, received_user_line):
                    print "FAILED: (%s) != (%s)" % (expected_user_line, received_user_line)
                    output[command] = "FAIL"
        output[command] = "PASSED"
    return output

if __name__ == "__main__":
    import optparse

    parser = optparse.OptionParser("usage: %prog [-d] <Test Module>")
    parser.add_option("-d", "--debug", dest="debug", action="store_const",
                      const=True, help="Turn on debugging")
    parser.add_option("-s", "--server_home", dest="server_home", metavar = "HOME",
                      help="Set server home")

    (options, modules_to_test) = parser.parse_args()
    debug = False
    if options.debug:
        debug = True

    system_state = SystemState()
    system_state.load_config()
    libUi.login("admin", 'abc123')

    output = system_state.cnm_connector.dispatcher_control("start")
    if not debug:
        print "(suppressing output)"
        output_handle = StringIO.StringIO()
        system_state.set_output(output_handle)
    else:
        print "OUTPUT FROM DISPATCHER START", output
        print "(clearing existing connections)"
    system_state.cnm_connector.cleanup_connections()

    if options.server_home:
        system_state.cnm_connector.sync_server_home(options.server_home)

    file_names = glob.glob("*.py")
    module_names = [ x.split('.')[0] for x in file_names ]
    module_names = [ n for n in module_names if n not in IGNORE_MODULE_NAMES ]

    for module_name in module_names:
        cls, setup_test = get_cls(module_name, modules_to_test)
        if not cls:
            continue
        if setup_test:
            setup_test()
        tests = get_tests(cls, module_name)
        if not tests:
            continue

        output = run_tests(cls, tests, debug)
        for command in output:
            print "%s:  '%s' (%s)" % (output[command], command, module_name)



#!/usr/bin/python
# BSD License
# Copyright (c) 2009, Peter Banka et al
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
# * Neither the name of the GE Security nor the names of its contributors may
#   be used to endorse or promote products derived from this software without
#   specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

""" PinshCmd for dispatcher service control and status
"""

__author__ =  'Peter Banka, Shawn Sherwood'
__version__ = '1.0'

import PinshCmd
from bombardier_core.static_data import OK, FAIL
from SystemStateSingleton import SystemState, ENABLE
from Exceptions import CommandError, ServerException
from Expression import Expression
system_state = SystemState()

class Dispatcher(PinshCmd.PinshCmd):
    '''bomsh# dispatcher status
       [OK, ['uptime: ==UNKNOWN==', 'active_jobs: []']]
       bomsh# dispatcher stop
       [OK, ['Dispatcher stopped']]
       bomsh# dispatcher stop
       [OK, ['Dispatcher stopped']]
       bomsh# dispatcher start
       [OK, ['Dispatcher started']]
       bomsh# dispatcher start
       [OK, ['Dispatcher already started']]
    '''

    def __init__(self):
        """Top-level object has a multiple choice child
        """
        PinshCmd.PinshCmd.__init__(self, "dispatcher")
        self.help_text = "dispatcher\tdispatcher control commands and status"
        self.cmd_owner = 1

        start = PinshCmd.PinshCmd("start", "start\tstart the dispatcher")
        stop = PinshCmd.PinshCmd("stop", "stop\tstop the dispatcher")
        status = PinshCmd.PinshCmd("status", "status\tstatus of the dispatcher")
        attach = PinshCmd.PinshCmd("attach", "attach\tattach to an already-running dispatcher")
        dispatcher_uri = Expression("uri")
        attach.children = [dispatcher_uri]

        self.children = [start, stop, status, attach]
        self.auth = ENABLE

    def cmd(self, tokens, no_flag):
        """
        tokens -- all of the keywords passed in the command string, parsed
        no_flag -- whether the 'no' keyword was used in the command string
        """
        action = tokens[1].lower()
        post_data = {}
        if action == "attach":
            if len(tokens) < 3:
                raise CommandError("Incomplete command")
            post_data["uri"]= tokens[2]
        return system_state.cnm_connector.dispatcher_control(action, post_data)




#!/usr/bin/python
# BSD License
# Copyright (c) 2009, Peter Banka et al
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
# * Neither the name of the GE Security nor the names of its contributors may
#   be used to endorse or promote products derived from this software without
#   specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

"""This provides a top-level command for the bomsh shell. Used to connect
directly from the CLI to a machine, bypassing the CNM."""

__author__ =  'Peter Banka'
__version__ = '1.0'

import PinshCmd
from ConfigField import ConfigField, MACHINE
from bombardier_core.static_data import OK, FAIL
from SystemStateSingleton import SystemState
system_state = SystemState()

import os

SSH = "/usr/bin/ssh"

class Ssh(PinshCmd.PinshCmd):
    def __init__(self):
        PinshCmd.PinshCmd.__init__(self, "ssh")
        self.help_text = "ssh\tssh to another host"
        self.config_field = ConfigField(data_type=MACHINE)
        self.children = [self.config_field]
        self.cmd_owner = 1
        self.log_command = True

    def cmd(self, tokens, no_flag):
        if no_flag:
            return FAIL, []
        if len(tokens) < 2:
            return FAIL, ["Incomplete command."]
        host_name = tokens[1]

        current_dict = self.config_field.get_specific_data(tokens, 1)
        ip_address = current_dict.get("ip_address")
        default_user = current_dict.get("default_user")

        if not default_user or not ip_address:
            msg  = ["ip_address and default_user are required in the system"]
            msg += ["configuration in order to access this host."]
            return FAIL, msg
        else:
            os.system('%s %s@%s'%(SSH,default_user,ip_address))
        return OK, []



""" JobNameField is a field that can become the name of a job
"""

import PinshCmd
import yaml
from bombardier_core.static_data import PARTIAL, COMPLETE, NO_MATCH
from SystemStateSingleton import SystemState, ENABLE

system_state = SystemState()

class JobNameField(PinshCmd.PinshCmd):
    def __init__(self, machine_name=None):
        "Allows a guy to find a job"
        PinshCmd.PinshCmd.__init__(self, "<job_name>")
        self.help_text = "<job_name>\tname of an active job"
        self.cmd_owner = 0
        self.machine_name = machine_name

    def get_jobs(self, server_output):
        data = yaml.load('\n'.join(server_output))
        active_jobs = data.get("active_jobs", [])
        return active_jobs

    def preferred_names(self, tokens, index):
        '''Provide a list of names that the system would prefer to use, other
        than that which was typed in by the user. For example, 'sho mach localh'
        will return 'localhost' for the machine name if strict is off,
        otherwise, it will return 'localh'.

        '''
        post_data = {}
        status, data = system_state.cnm_connector.dispatcher_control("status",
                                                                    post_data)
        if not data:
            return []
        job_names = data.get("active_jobs", [])
        job_names += data.get("broken_jobs", [])
        job_names += data.get("finished_jobs", [])
        tokens[index] = tokens[index].replace('"', '')
        if len(job_names) == 0:
            return []
        if self.machine_name:
            new_job_names = []
            matcher = re.compile("(\S+)\@(\S+)\-(\d+)")
            for job_name in job_names:
                _user, machine_name, _job_id = matcher.findall(job_name)
                if machine_name == self.machine_name:
                    new_job_names.append(job_name)
            job_names = new_job_names

        possible_names = []
        for job_name in job_names:
            if job_name.startswith(tokens[index]):
                possible_names.append(job_name)
        return possible_names

    def match(self, tokens, index):
        '''Determines if what has been typed in by the user matches a
        configuration item that the system is keeping track of.'''
        possible_matches = self.acceptable_names(tokens, index)
        if not possible_matches:
            return NO_MATCH, 1
        if len(possible_matches) > 1:
            return PARTIAL, 1
        return COMPLETE, 1





#!/usr/bin/python
# BSD License
# Copyright (c) 2009, Peter Banka et al
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
# * Neither the name of the GE Security nor the names of its contributors may
#   be used to endorse or promote products derived from this software without
#   specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

'''A [PinshCmd] command which allows a user to create a new data item
on the server'''

import yaml, tempfile, os, libUi
import PinshCmd, ConfigField
from bombardier_core.static_data import OK, FAIL
from SystemStateSingleton import SystemState
system_state = SystemState()

class CreateType(PinshCmd.PinshCmd):
    '''A thing that can be created.'''
    def __init__(self, name, help_text):
        PinshCmd.PinshCmd.__init__(self, name, help_text)
        self.config_field = None
        self.cmd_owner = 1

    def cmd(self, tokens, no_flag):
        '''Create a thing that the user is interested in'''
        if no_flag:
            return FAIL, []
        if len(tokens) < 3:
            return FAIL, ["Incomplete command."]

        conf_str = self.config_field.get_default_data()
        #conf_str = yaml.dump(current_dict, default_flow_style=False)
        fd,fn = tempfile.mkstemp(suffix=".yml", text=True)
        fh = os.fdopen(fd, 'w+b')
        fh.write(conf_str)
        fh.close()
        os.system("vim %s" % fn)
        post_data = yaml.load(open(fn).read())
        submit = libUi.ask_yes_no("Commit changes to server", libUi.YES)
        if submit:
            status, output = self.config_field.post_specific_data(tokens, 2, post_data)
            os.unlink(fn)
            return OK, [output]
        else:
            msg = "Discarded changes. Edits can be found here: %s" % fn
            return OK, [msg]

class Machine(CreateType):
    'Create a default machine configuration data from the server'
    def __init__(self):
        CreateType.__init__(self, "machine", "machine\tcreate a new machine configuration")
        self.config_field = ConfigField.ConfigField(data_type=ConfigField.MACHINE, new=True)
        self.children = [self.config_field]

class Include(CreateType):
    'Create an include file on the server'
    def __init__(self):
        CreateType.__init__(self, "include", "include\tcreate a shared include file")
        self.config_field = ConfigField.ConfigField(data_type=ConfigField.INCLUDE, new=True)
        self.children = [self.config_field]

class Package(CreateType):
    'Create a package file'
    def __init__(self):
        CreateType.__init__(self, "package", "package\tcreate new package metadata")
        self.config_field = ConfigField.ConfigField(data_type=ConfigField.PACKAGE, new=True)
        self.children = [self.config_field]

class Bom(CreateType):
    'create new bill-of-materials ("bom") file from the server'
    def __init__(self):
        CreateType.__init__(self, "bom", "bom\tcreate a bill of materials")
        self.config_field = ConfigField.ConfigField(data_type=ConfigField.BOM, new=True)
        self.children = [self.config_field]

class Create(PinshCmd.PinshCmd):
    '''Create a file'''
    def __init__(self):
        PinshCmd.PinshCmd.__init__(self, "create")
        self.help_text = "create\tcreate a new system component"
        machine = Machine()
        include = Include()
        package = Package()
        bom = Bom()
        self.children = [machine, include, bom, package]
        self.cmd_owner = 1



#!/usr/bin/python
# BSD License
# Copyright (c) 2009, Peter Banka et al
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
# * Neither the name of the GE Security nor the names of its contributors may
#   be used to endorse or promote products derived from this software without
#   specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

'''A [PinshCmd] object that provides a field with automated completion.
For stuff in a machine's status field'''

import PinshCmd
from bombardier_core.static_data import PARTIAL, COMPLETE, NO_MATCH
from SystemStateSingleton import SystemState
from Exceptions import MachineStatusException
system_state = SystemState()

FIX=1
PURGE=2
INSTALLED=3
NOT_INSTALLED=4

def clean_version_numbers(installed_set):
    output = set()
    for package_name in installed_set:
        fields = package_name.split('-')
        basic_package_name = '-'.join(fields[:-1])
        output.update([basic_package_name])
    return output

class PackageField(PinshCmd.PinshCmd):
    '''Some times you want to maintain package status data on a machine.'''
    def __init__(self, action_type):
        '''
        action_type -- FIXABLE or PURGABLE
        machine_name_index -- which token in the set of arguments passed
                              to preferred_names is the machine name
        package_name_index -- which token is the package_name to be completed
        '''
        PinshCmd.PinshCmd.__init__(self, "PackageField")
        self.help_text = "<PackageField>\t"\
                         "A package name that comes from machine status"
        self.action_type = action_type
        self.cmd_owner = 0
        self.machine_name_index = 1

    def possible_names(self, machine_name):
        'returns a list of all self.data_type things'
        url = "json/summary/name/%s" % (machine_name)
        data = system_state.cnm_connector.service_yaml_request(url)
        #print "PN: data:" % data
        if data.get("command_status") == "FAIL":
            raise MachineStatusException(data.get("command_output"))
        broken = set(data.get("broken", []))
        not_installed = set(data.get("not_installed", []))
        full_installed = set(data.get("installed", []))
        installed = clean_version_numbers(full_installed)
        #print "action_type: %s" % self.action_type
        if self.action_type == FIX:
            return list(broken.union(not_installed))
        if self.action_type == PURGE:
            return list(broken.union(full_installed))
        if self.action_type == INSTALLED:
            return list(installed)
        if self.action_type == NOT_INSTALLED:
            return list(not_installed)

    def get_package_names(self, complete_machine_name,
                           incomplete_package_name):
        '''someone typed in something like 'show machine localho'. Our job is
        to figure out that localhost is the object that needs to be found and
        to return the dictionary for that object.'''
        possible_names = self.possible_names(complete_machine_name)
        #print "OBJECT NAMES:",possible_names
        package_names = []
        for possible_name in possible_names:
            if possible_name.lower().startswith(incomplete_package_name.lower()):
                #print "%s starts with the same as incomplete_package_name: %s" % (possible_name, incomplete_package_name)
                package_names.append(possible_name)
        if len(package_names) == 0:
            return []
        if incomplete_package_name in package_names:
            package_names = [incomplete_package_name]
        if len(package_names) > 1:
            return package_names
        package_name = package_names[0]
        return [package_name]

    def preferred_names(self, tokens, index):
        '''Provide a list of names that the system would prefer to use, other
        than that which was typed in by the user. For example, 'machine local status fix package-1-1'
        will return 'localhost' for the machine name if strict is off,
        otherwise, it will return 'localh'.'''
        #print "PN: Tokens being passed (%s)" % tokens
        complete_machine_name = tokens[self.machine_name_index]
        incomplete_package_name = tokens[index]
        package_names = self.get_package_names(complete_machine_name,
                                               incomplete_package_name)
        return package_names

    def match(self, tokens, index):
        '''Determines if what has been typed in by the user matches a
        configuration item that the system is keeping track of.'''
        possible_matches = self.acceptable_names(tokens, index)
        if not possible_matches:
            return NO_MATCH, 1
        if len(possible_matches) > 1:
            return PARTIAL, 1
        return COMPLETE, 1



#!/usr/bin/env python

# BSD License
# Copyright (c) 2009, Peter Banka et al
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
# * Neither the name of the GE Security nor the names of its contributors may
#   be used to endorse or promote products derived from this software without
#   specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

'''A set of routines that handles user input/output on the terminal'''

import sys, termios, tty
from CnmConnector import CnmConnector, UnexpectedDataException, ServerException
from SystemStateSingleton import SystemState
system_state = SystemState()
from bombardier_core.static_data import OK, FAIL
from bombardier_core.static_data import WARNING, ERROR, CRITICAL, DEBUG, INFO
from bombardier_core.static_data import USER, ADMIN, LOG_LEVEL_LOOKUP
from bombardier_core.static_data import GOOD_COLOR, WARNING_COLOR, STRONG_COLOR
from bombardier_core.static_data import WEAK_COLOR, NO_COLOR

import re


ONE_LINE = 0
QUIT = 1
BACKUP = 2
PAGE = 3
NEUTRAL = 5
YES = 1
NO = 0

def login(username, password=None):
    'Prompt for a password and initialize connection to the server'
    system_state.get_term_info()
    if username:
        system_state.username = username
    if not system_state.username:
        try:
            system_state.username = get_default("username", "root")
        except KeyboardInterrupt:
            print "\n%% Aborted login"
            sys.exit(1)
    else:
        print "Logging in as %s" % system_state.username
    system_state.cnm_connector = CnmConnector(system_state.cnm_url,
                                 system_state.username)
    tries = 0
    while tries < 3:
        if not password:
            password = pwd_input("password: ")
        try:
            superuser_status = system_state.cnm_connector.login(password)
            if superuser_status:
                system_state.set_auth(ADMIN)
            else:
                system_state.set_auth(USER)
            break
        except UnexpectedDataException, ude:
            print ude
            user_output(["Bad username or password."], FAIL)
            tries += 1
        except ServerException, sex:
            user_output(["Server not accessible (%s)" % sex], FAIL)
            sys.exit(1)
        password = ''
    if tries >= 3:
        user_output(["Access denied."], FAIL)
        sys.exit(1)
    system_state.prompt = ['>']
    system_state.set_prompt()

def motd():
    'Print out the message of the day after login'
    from banner import banner
    for line in banner:
        system_state.fp_out.write(line)
        system_state.fp_out.flush()

def append_not_blank(current_token, tokens):
    '''append something to the list if it isn't blank. Handles users typing
    in more than one space between tokens'''
    current_token = current_token.strip()
    if current_token:
        tokens.append(current_token)
    return tokens

def tokenize(input_str):
    '''Takes an input string and divides it into string tokens. Handles
    the word "no" in front of the string specially, handles a '?' specially
    handles quoted text as one unit, and handles comment markers'''
    tokens = []
    quote_mode = False
    current_token = ''
    append_last = False
    comment = ''
    for i in range(0, len(input_str)):
        char = input_str[i]
        if not quote_mode:
            if char not in ['"', '#', ' ']:
                current_token += char
                append_last = True
                continue
            tokens = append_not_blank(current_token, tokens)
            if char == '"': # start quote
                quote_mode = True
            elif char == '#': # discard the rest, it's a real comment
                if i != 0 and input_str[i-1] == ' ':
                    append_last = True
                else:
                    append_last = False
                comment = input_str[i+1:]
                break
            elif char == ' ': # tokenize on spaces if not quoted
                current_token = ''
                append_last = True
        else:
            if char == '"': # end quote
                tokens = append_not_blank(current_token, tokens)
                current_token = ''
                append_last = False
                quote_mode = False
            else:
                current_token += char
    if quote_mode: # unbalanced quotes
        raise Exception
    if append_last:
        tokens.append(current_token.lstrip()) # grab the last
    return tokens, comment

def process_input(string):
    '''take a line of input from the user and convert it into an argv
    type structure'''
    if len(string) == 0:
        return 0, 0, [], ''
    # determine if the help_flag is there (cheap hack)
    help_flag = 0
    strlen = len(string)
    if strlen >= 1 and string[-1] == '?':
        help_flag = 1
        string = string[:-1]
    if len(string) == 0:
        return 0, 1, [], ''
    string.lstrip() # left space is unimportant, right space is important
    tokens, comment = tokenize(string)
    # handle a preceding 'no'
    no_flag = 0
    if not tokens:
        return 0, 0, [], comment
    if tokens[0] == 'no':
        no_flag = 1
        if len(tokens) > 1:
            tokens = tokens[1:]
        else:
            return 0, 0, [], comment
    # get rid of extra spaces, except at the end!
    processed_data = []
    for token in tokens:
        if token != '':
            processed = False
            if token.startswith('$'):
                var_name = token[1:]
                if system_state.globals.has_key(var_name):
                    value = system_state.globals.get(var_name)
                    if type(value) == type(["list"]):
                        processed_data.append('[')
                        processed_data += value
                        processed_data.append(']')
                    else:
                        token = str(value)
                    processed = True
            if token.startswith('[') and len(token) > 1:
                processed_data.append('[')
                token = token[1:]
            if token.endswith(']') and len(token) > 1:
                processed_data.append(token[:-1])
                processed_data.append(']')
                processed = True
            if not processed:
                processed_data.append(token)
    if tokens[-1] != '':
        tokens = processed_data
    else:
        tokens = processed_data + ['']
    if len(tokens) == 0:
        return 0, 0, [], comment
    if tokens[0] == "ls" and len(tokens) == 1:
        tokens[0] = ''
        help_flag = True
    return no_flag, help_flag, tokens, comment

# User Input
def get_default(prompt, default):
    'prompts the user for a question and accepts a default value'
    instr = raw_input(prompt+" ["+default+"]: ")
    if instr == "":
        instr = default
    return instr

def pwd_input(prompt):
    'ask for a password, providing asterisks for output'
    file_handle = sys.stdin.fileno()
    old_settings = termios.tcgetattr(file_handle)
    try:
        tty.setraw(sys.stdin.fileno())
        print prompt,
        password = ''
        while 1 == 1:
            char = sys.stdin.read(1)
            if char == chr(3): # ^C was pressed
                raise KeyboardInterrupt
            if char == chr(13): # Enter was pressed
                break
            if char == chr(8) or char == chr(127): # backspace
                if len(password) > 0:
                    sys.stdout.write("\b")
                    sys.stdout.write(" ")
                    sys.stdout.write("\b")
                    password = password[:-1]
                continue
            if ord(char) > 31 and ord(char) < 128: # Valid character
                sys.stdout.write("*")
                sys.stdout.flush()
                password += char
    except KeyboardInterrupt:
        sys.exit(1)
    finally:
        termios.tcsetattr(file_handle, termios.TCSADRAIN, old_settings)
    redaction = "\b" * len(password) + " " * len(password)
    sys.stdout.write(redaction)
    print
    return password

def get_password(prompt = "enter password:"):
    'have the user enter a password and verify it'
    while 1 == 1:
        password1 = pwd_input(prompt)
        password2 = pwd_input("Please re-enter password:")
        if password1 == password2:
            return password1
        print "% Passwords do not match\n\r"

def ask_yes_no(prompt, default = NEUTRAL):
    'ask a yes or no question from the user'
    if default == NEUTRAL:
        prompt += "? (y/n): "
    elif default == YES:
        prompt += "? (Y/n): "
    elif default == NO:
        prompt += "? (y/N): "
    else:
        print prompt+": YES"
        return YES
    result = NEUTRAL
    while result == NEUTRAL:
        result = default
        try:
            instr = raw_input(prompt)
        except EOFError:
            continue
        if len(instr) == 0:
            continue
        if instr.lower()[0] == 'y':
            result = YES
        elif instr.lower()[0] == 'n':
            result = NO
    return result

def page_it():
    "asks for the user to hit enter or 'b' to backup"
    system_state.fp_out.write("--- more ---")
    system_state.fp_out.flush()
    file_handle = sys.stdin.fileno()
    old_settings = termios.tcgetattr(file_handle)
    try:
        tty.setraw(sys.stdin.fileno())
        char = sys.stdin.read(1)
    finally:
        termios.tcsetattr(file_handle, termios.TCSADRAIN, old_settings)
    system_state.fp_out.write("\r                \r")
    system_state.fp_out.flush()
    if char == chr(13):
        return ONE_LINE
    if char == chr(113) or char == chr(81):
        return QUIT
    if char == chr(98) or char == chr(66):
        return BACKUP
    return PAGE

def pager_out(print_data):
    "pages output to the screen if there's more than the terminal will handle"
    if system_state.termlen == 0:
        print print_data
        return
    current_line = 0
    pager_line = 0
    backup = 0
    print_data = print_data.split("\n")
    while current_line < len(print_data):
        system_state.fp_out.write(print_data[current_line]+"\n")
        system_state.fp_out.flush()
        current_line += 1
        pager_line += 1
        if not system_state.batch and pager_line > system_state.termlen:
            action = page_it()
            if action == QUIT:
                break
            elif action == ONE_LINE:
                backup = 0
                continue
            elif action == BACKUP:
                backup_len = (backup + 1) * system_state.termlen + 1
                if backup_len > current_line:
                    backup_len = current_line
                system_state.fp_out.write("\n\n-- backing up %d lines -- \n\r" % backup_len)
                system_state.fp_out.flush()
                current_line -= backup_len
                backup += 1
                pager_line = 0
                continue
            else:
                backup = 0
                pager_line = 0

def prepender(prepend, obj, depth):
    "output formatter for indented text"
    print_data = ''
    if type(obj) == type(['list']):
        for entry in obj:
            print_data += prepender(prepend, entry, depth+1)
    elif type(obj) == type({}):
        for entry in obj:
            if type(obj[entry]) == type(['list']) and len(obj[entry]) == 0:
                print_data += '%s%s%s: []\n' % (prepend, ' '*depth, entry)
            elif type(obj[entry]) == type('string'):
                print_data += '%s%s%s: \'%s\'\n' % (prepend, ' '*depth, entry, obj[entry])
            elif type(obj[entry]) in [ type(0), type(0.1) ]:
                print_data += '%s%s%s: %d\n' % (prepend, ' '*depth, entry, obj[entry])
            elif type(obj[entry]) == type({}) and len(obj[entry]) == 0:
                print_data += '%s%s%s: {}\n' % (prepend, ' '*depth, entry)
            else:
                print_data += '%s%s%s:\n' % (prepend, ' '*depth, entry)
                print_data += prepender(prepend, obj[entry], depth+1)
    elif type(obj) in [ type('string'), type(u'unicode') ]:
        print_data += "%s%s%s\n" % (prepend, ' '*depth, obj)
    elif type(obj) in [ type(0), type(0.1) ]:
        print_data += "%s%s%d\n" % (prepend, ' '*depth, obj)
    return print_data

# FIXME: CONSOLIDATE LOGGING HERE

def info(msg):
    'logging function'
    system_state.fp_out.write(">>> %s\n" % msg)

def debug(msg):
    'logging function'
    if system_state.debug == True:
        system_state.fp_out.write(">>> DEBUG: %s\n" % msg)

def warning(msg):
    'logging function'
    system_state.fp_out.write(">>> WARNING: %s\n" % msg)

def error(msg):
    'logging function'
    system_state.fp_out.write(">>> ERROR: %s\n" % msg)

def process_cnm(server_output_lines):
    'Handles output from the CNM and pretty-prints to the screen'
    for line in server_output_lines:
        line = line.strip()
        if line.endswith('|'):
            line = line[:-1]
        if not line:
            continue
        components = line.split('|')
        if len(components) >= 4:
            log_level_string = components[1]
            log_level = LOG_LEVEL_LOOKUP.get(log_level_string, CRITICAL)
            if log_level >= system_state.log_level:
                date = components[0]
                job_name = components[2]
                if system_state.termcolor != NO_COLOR:
                    color_code = None
                    if log_level == DEBUG:
                        color_code = WEAK_COLOR[system_state.termcolor]
                    elif log_level == WARNING:
                        color_code = STRONG_COLOR[system_state.termcolor]
                    elif log_level > WARNING:
                        color_code = WARNING_COLOR[system_state.termcolor]
                    elif log_level != INFO:
                        print "=== UNKNOWN log level: ", log_level
                        color_code = WARNING_COLOR[system_state.termcolor]
                    if color_code != None:
                        prefix = "  | %20s | " % (job_name)
                        message = "%s\033%s%s\033[m\n" % (prefix, color_code, '|'.join(components[3:]))
                    else:
                        message = "  | %20s | %s\n" % (job_name, '|'.join(components[3:]))
                else:
                    message = "  | %20s | %s\n" % (job_name, '|'.join(components[3:]))
                system_state.fp_out.write(message)
        else:
            system_state.fp_out.write("-- %s\n" % line )


def user_output(output, status, prepend = '', test = False):
    "Generic function for printing return values from a command"
    if output == []:
        return
    if prepend == '':
        if status == FAIL:
            prepend = '% '
        else:
            prepend = ' '
    if test:
        return prepender(prepend, output, 0)
    pager_out(prepender(prepend, output, 0))
    return

def process_traceback(machine_exception):
    output = {"Exception": str(machine_exception),
              "data": machine_exception.traceback}
    user_output(output, FAIL)

def user(string):
    "just print some text to stderr for user feedback"
    print >> sys.stderr
    print >> sys.stderr, string



#!/usr/bin/env python

# BSD License
# Copyright (c) 2009, Peter Banka et al
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
# * Neither the name of the GE Security nor the names of its contributors may
#   be used to endorse or promote products derived from this software without
#   specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

'''A [PinshCmd] that is used to keep track of terminal settings.'''

import PinshCmd, Integer
import MultipleChoice
from bombardier_core.static_data import NO_COLOR, LIGHT, DARK, OK, FAIL, COMPLETE
from SystemStateSingleton import SystemState
system_state = SystemState()

class Terminal(PinshCmd.PinshCmd):
    'If you want to set your terminal, use this'
    def __init__(self):
        PinshCmd.PinshCmd.__init__(self, "terminal")
        self.help_text = "terminal\tchange settings for the current terminal"
        self.length = PinshCmd.PinshCmd("length", "length\tnumber of lines the terminal can display")
        self.width  = PinshCmd.PinshCmd("width", "width\tnumber of columns the terminal can disply")
        self.color  = PinshCmd.PinshCmd("color", "color\tset terminal color")
        help_text = ["black and white", "colors for light background", "colors for dark background"]
        color_options = MultipleChoice.MultipleChoice(choices = ["none", "light", "dark"],
                                                     help_text = help_text)
        self.children = [self.length, self.width, self.color]
        self.num = Integer.Integer(min_value = 0, max_value = 1000, name = "<int>")
        self.length.children = [self.num]
        self.width.children = [self.num]
        self.color.children = [color_options]
        self.cmd_owner = 1

    def cmd(self, tokens, no_flag):
        'Someone wants to execute a command to change their terminal'
        if no_flag:
            if tokens[1].lower().startswith('c'):
                system_state.termcolor = NO_COLOR
                return OK, ["Terminal set to black and white."]
            else:
                return FAIL, ["'No' not applicable in this context"]
        if len(tokens) < 3:
            msg  = ["terminal length: %d" % system_state.termlen]
            msg += ["terminal width: %d" % system_state.termwidth]
            if system_state.termcolor != NO_COLOR:
                if system_state.termcolor == LIGHT:
                    msg += ["terminal color-scheme: light"]
                if system_state.termcolor == DARK:
                    msg += ["terminal color-scheme: dark"]
            return OK, msg
        value = tokens[2]
        if tokens[1].lower().startswith('c'):
            if value.lower().startswith('n'):
                system_state.termcolor = NO_COLOR
                return OK, ["Terminal set to black and white."]
            elif value.lower().startswith('l'):
                system_state.termcolor = LIGHT
                return OK, ["Terminal set to light background."]
            elif value.lower().startswith('d'):
                system_state.termcolor = DARK
                return OK, ["Terminal set to dark background."]
            else:
                return FAIL, ["Unknown terminal color."]

        if self.num.match([value], 0) != (COMPLETE, 1):
            return FAIL, ["Please choose a value between 0 and 1000"]

        if tokens[1].lower().startswith('l'):
            system_state.termlen = int(value)
        elif tokens[1].lower().startswith('w'):
            system_state.termwidth = int(value)
        return OK, []



# BSD License
# Copyright (c) 2009, Peter Banka et al
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
# * Neither the name of the GE Security nor the names of its contributors may
#   be used to endorse or promote products derived from this software without
#   specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

"""Provides utilities for accessing the RESTful web service, which is the
new core of the Bombardier server."""

__author__ =  'Peter Banka'
__version__ = '1.0'

import StringIO
import urllib2, urlparse, os, time
import yaml
import pycurl
import urllib
from bombardier_core.static_data import OK, FAIL
from Exceptions import UnexpectedDataException, ServerException
from Exceptions import MachineTraceback, ServerTracebackException
from Exceptions import MachineUnavailableException
import libUi
from SystemStateSingleton import SystemState, ENABLE
system_state = SystemState()

LOGIN_PATH = "/accounts/login/"

def make_query_string(args):
    """args -- a dictionary.
    Converts a dictionary into HTML keyword arguments for

    >>> make_query_string({'a': 'hello', 'b':'there'})
    '?a=hello&b=there'
    """
    query_string = ""
    if len(args.keys()) >= 1:
        for arg_index in range(0, len(args.keys())):
            key = args.keys()[arg_index]
            value = args[args.keys()[arg_index]]
            if arg_index == 0:
                query_string += "?%s=%s" % (key, value)
            else:
                query_string += "&%s=%s" % (key, value)
    return query_string

class Storage:
    '''An object used to hold the output of a pyCurl command'''
    def __init__(self):
        self.contents = []

    def store(self, buf):
        '''Command used by pycurl to store data
        buf -- content to add '''
        self.contents.append(buf)

    def __str__(self):
        return '\n'.join(self.contents)

class Header(Storage):
    '''A specific kind of storage object which holds HTML header information'''
    def convert_from_yaml(self):
        '''Since a header is largely YAML, this simply converts it to a
        dictionary'''
        yaml_lines = [ line for line in list(self.contents) if ':' in line ]
        yaml_string = ''.join(yaml_lines)
        data = yaml.load(yaml_string)
        return data

    def get_content_type(self):
        '''Returns the self-described content type'''
        data = self.convert_from_yaml()
        return data.get("Content-Type")

class Response:
    '''Aggregation of Header and Storage objects to form an HTTP response'''
    def __init__(self, header, output):
        '''header -- Header object
        output -- Storage object (for body)
        '''
        self.header = header
        self.output = output
        self.http_code = None

    def set_http_code(self, http_code):
        '''setter for http response code
        http_code -- HTTP response code from server
        '''
        self.http_code = http_code

    def convert_from_yaml(self):
        '''Assumes that the server attempted to respond with YAML(JSON).
        Verifies the content type and then converts the body'''
        if self.header.get_content_type() == "application/json":
            try:
                output = yaml.load(str(self.output))
            except yaml.parser.ParserError:
                # New Django JSON is not compatible with yaml. Massage it.
                try:
                    new_output = ''.join(str(self.output).split())
                    output = yaml.load(new_output)
                except yaml.parser.ParserError:
                    msg = "can't convert to yaml (%s)" % self.output
                    raise UnexpectedDataException(msg)
            if type(output) != type([]) and type(output) != type({}):
                raise UnexpectedDataException("Not a list or dictionary")
            if output == None:
                raise UnexpectedDataException("Null value received")
            if 'traceback' in output:
                raise ServerTracebackException(output['traceback'])
            return output
        raise UnexpectedDataException("can't convert to yaml")

    def __str__(self):
        return str(self.output)

class CnmConnector:
    '''Object to make it easier to communicate with the Bombardier RESTFul
    web server'''

    def __init__(self, address, username):
        '''
        address -- URL of the Bombardier web server
        username -- username for connecting to the server'''
        self.address = address
        self.username = username
        self.password = None
        self.proxy_address = None
        self.proxy_port = None
        self.debug = False
        self.session_id = None
        self.cookie_file = os.path.join(os.environ.get("HOME"), '.bom_cookie.txt')

    def prepare_curl_object(self, url):
        '''prepares pycurl object that can be used to communicate to the
        Bombardier web service
        url -- full path of the destination
        '''
        curl_obj = pycurl.Curl()
        curl_obj.setopt(pycurl.URL, url)
        curl_obj.setopt(pycurl.SSL_VERIFYPEER, 0)
        curl_obj.setopt(pycurl.FAILONERROR, 1)
        curl_obj.setopt(pycurl.FOLLOWLOCATION, 1)
        curl_obj.setopt(pycurl.COOKIEFILE, self.cookie_file)
        curl_obj.setopt(pycurl.COOKIEJAR, self.cookie_file)
        if self.debug:
            print "CONNECTING TO: ",url
        if self.username and self.password:
            auth_string = "%s:%s" % (self.username, self.password)
            curl_obj.setopt(pycurl.USERPWD, auth_string)
            #curl_obj.setopt(pycurl.HTTPAUTH, pycurl.HTTPAUTH_ANYSAFE)
            curl_obj.setopt(pycurl.HTTPAUTH, pycurl.HTTPAUTH_BASIC)
        if self.proxy_address != None:
            curl_obj.setopt(pycurl.PROXY, self.proxy_address)
            if self.proxy_port != None:
                curl_obj.setopt(pycurl.PROXYPORT, self.proxy_port)
        return curl_obj

    def perform_request(self, curl_obj, full_path):
        '''Connects to the web server and prepares returns a response
        object.
        curl_obj -- a pycurl object, initialized
        full_path -- the full destination of connection
        '''
        output = Storage()
        header = Header()
        curl_obj.setopt(pycurl.WRITEFUNCTION, output.store)
        curl_obj.setopt(pycurl.HEADERFUNCTION, header.store)
        response = Response(header, output)
        if self.debug:
            print "PERFORMING:", curl_obj
        try:
            curl_obj.perform()
            http_code = curl_obj.getinfo(pycurl.HTTP_CODE)
            response.set_http_code(http_code)
            return response
        except pycurl.error, curl_err:
            http_code = curl_obj.getinfo(pycurl.HTTP_CODE)
            raise ServerException(full_path, curl_err[1], http_code)

    def post(self, path, data):
        '''Prepare to perform a POST
        path -- relative URL for posting
        data -- dictionary of POST data
        '''
        full_path = urlparse.urljoin(self.address, path)
        curl_obj = self.prepare_curl_object(full_path)
        post_data = []
        for key in data:
            post_data.append((key, data[key]))
        encoded_post_data = urllib.urlencode(post_data)
        curl_obj.setopt(pycurl.POSTFIELDS, encoded_post_data)
        if self.debug:
            print "POSTING:", encoded_post_data
        return self.perform_request(curl_obj, full_path)

    def login(self, password):
        '''Log in to the web server. Saves a sessionid
        password -- cleartext password to use to authenticate
        '''
        data = {"username": self.username, "password": password}
        response = self.post(LOGIN_PATH, data)
        return_data = response.convert_from_yaml()
        self.password = password
        return return_data.get("super_user")

    def service_request(self, path, args=None,
                        put_data=None, post_data=None, timeout=None):
        '''Performs a GET or a POST or a PUT, depending on data provided
        and returns text data based on output
        path -- relative path
        args -- query string variables to set
        put_data -- data to convert to a PUT
        post_data -- data to convert to a POST
        '''
        if not args:
            args = {}
        full_path = urlparse.urljoin(self.address, path)
        query_string = make_query_string(args)
        url = urlparse.urljoin(full_path, query_string)
        if self.debug:
            print "Performing service request to %s" % url
        curl_obj = self.prepare_curl_object(url)
        if timeout != None:
            curl_obj.setopt(pycurl.TIMEOUT, timeout)
            if self.debug:
                print "Setting timeout: ", timeout
        if put_data != None:
            curl_obj.setopt(pycurl.UPLOAD, 1)
            curl_obj.setopt(pycurl.INFILESIZE, len(put_data))
            out_data = StringIO.StringIO(put_data)
            curl_obj.setopt(pycurl.READFUNCTION, out_data.read)
            if self.debug:
                print "Setting PUT data"
        if post_data != None:
            post_list = []
            for key in post_data:
                post_list.append((key, post_data[key]))
            encoded_post_data = urllib.urlencode(post_list)
            curl_obj.setopt(pycurl.POSTFIELDS, encoded_post_data)
            curl_obj.setopt(pycurl.POST, 1)
            if self.debug:
                print "POSTING:", encoded_post_data
        return self.perform_request(curl_obj, full_path)

    def sync_server_home(self, server_home_path):
        url = '/json/server/config'
        post_data={"server_home": server_home_path}
        output = self.service_yaml_request(url, post_data=post_data)
        new_path = output["server_home"]
        if new_path != server_home_path:
            raise UnexpectedDataException("server_home is set to %s" % new_path)

    def service_yaml_request(self, path, args=None,
                             put_data=None, post_data=None, timeout=None):
        '''same as service_request, but assumes that return data from the server
        is YAML(JSON) and converts it to return a python dictionary instead of text.'''
        try:
            if put_data != None:
                if type(put_data) == type(["list"]) or \
                   type(put_data) == type({}):
                    put_data = yaml.dump(put_data)
            response = self.service_request(path, args, put_data, post_data, timeout)
            return response.convert_from_yaml()
        except urllib2.HTTPError:
            print "Unable to connect to the service %s" % path
            return {}

    def cleanup_connections(self):
        url = "json/machine/cleanup_connections"
        self.service_yaml_request(url, post_data={})

    def _is_any_alive(self, poll_output):
        for job_name in poll_output:
            if poll_output[job_name].get("alive", False):
                return True
        return False

    def watch_jobs(self, job_names):
        libUi.info("Job names: %s" % job_names)
        summary_output = {}
        while job_names:
            time.sleep(0.25)
            url = "json/job/poll/"
            post_data = {"yaml": yaml.dump({"job_names": job_names})}
            output = system_state.cnm_connector.service_yaml_request(url, post_data=post_data)
            for job_name in output:
                job_output = output[job_name]
                if "traceback" in job_output:
                    print "TRACEBACK"
                    raise MachineTraceback(url, job_output["traceback"]) #FIXME what happens here?
                new_output = job_output.get("new_output", '')
                if new_output:
                    libUi.process_cnm(new_output)
                if not job_output.get("alive", False):
                    libUi.info("Joining %s..." % job_name)
                    out = system_state.cnm_connector.join_job(job_name)
                    summary_output[job_name] = out
                    job_names +=  out.get("children", [])
                    job_names.remove(job_name)
                 
        return summary_output

    def get_job(self, url, post_data):
        out = self.service_yaml_request(url, post_data=post_data)
        if "traceback" in out:
            raise MachineTraceback(url, out["traceback"])
        if out.get("command_status") != OK:
            raise MachineUnavailableException(out.get("command_output",""))
        job_name = out.get("job_name")
        return job_name

    def join_job(self, job_name):
        url = "json/job/join/%s" % job_name
        output = self.service_yaml_request(url)
        if "traceback" in output:
            raise MachineTraceback(url, out["traceback"])
        return output

    def set_password(self, password):
        url = "/json/dispatcher/set-password"
        post_data = {"password": password}
        output = self.service_yaml_request(url, post_data=post_data)
        return output["command_status"], output["command_output"]

    def dispatcher_control(self, action, post_data = {}):
        dispatcher_url = "/json/dispatcher/%s" % action
        if action == "status":
            output = self.service_yaml_request(dispatcher_url)
            if output.get('command_status') == OK:
                return OK, output
            else:
                return_list = ["Dispatcher is offline."]
                return_status = FAIL
            return return_status, return_list
        else:
            try:
                output = self.service_yaml_request(dispatcher_url, 
                                                   post_data = post_data,
                                                   timeout=5)
            except ServerException:
                if action == "start":
                    output = {"command_status": OK,
                              "command_output": ['Dispatcher started']}
                else:
                    raise
        return output["command_status"], output["command_output"]



#!/usr/bin/python
# BSD License
# Copyright (c) 2009, Peter Banka et al
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
# * Neither the name of the GE Security nor the names of its contributors may
#   be used to endorse or promote products derived from this software without
#   specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

"Provides a nice way to exit the bomsh (^D also works)"

__author__ =  'Peter Banka'
__version__ = '1.0'

import sys

import PinshCmd

class Exit(PinshCmd.PinshCmd):
    'Exit the bomsh'
    def __init__(self):
        PinshCmd.PinshCmd.__init__(self, "exit")
        self.help_text = "exit\texit current mode"
        self.cmd_owner = 1

    def cmd(self, tokens, noFlag):
        sys.exit(0)



#!/usr/bin/python
# BSD License
# Copyright (c) 2009, Peter Banka et al
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
# * Neither the name of the GE Security nor the names of its contributors may
#   be used to endorse or promote products derived from this software without
#   specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

'''A [PinshCmd] command which allows a user to display data'''


import readline
import yaml
import PinshCmd, ConfigField, Integer
from bombardier_core.static_data import OK, FAIL
from SystemStateSingleton import SystemState
system_state = SystemState()

TERM_OVERCOUNT = 8 # For some reason, the term width seems too long...

class ShowType(PinshCmd.PinshCmd):
    '''A thing that can be shown.'''
    def __init__(self, name, help_text):
        PinshCmd.PinshCmd.__init__(self, name, help_text)
        self.config_field = None
        self.cmd_owner = 1

    def cmd(self, tokens, no_flag):
        '''Show the thing that the user is interested in'''
        if no_flag:
            return FAIL, []
        if len(tokens) < 3:
            return FAIL, ["Incomplete command."]
        current_dict = self.config_field.get_specific_data(tokens, 2)
        return OK, yaml.dump(current_dict, default_flow_style=False).split('\n')

class Merged(ShowType):
    '''Merged objects take all the configuration for a machine and show it to
    the user, folding in include and bom data'''
    def __init__(self):
        ShowType.__init__(self, "merged", "merged\tdisplay a merged configuration")
        self.config_field = ConfigField.ConfigField()
        self.children = [self.config_field]

class Status(ShowType):
    'Displays status of a machine'
    def __init__(self):
        ShowType.__init__(self, "status", "status\tshows installation status")
        self.config_field = ConfigField.ConfigField(data_type=ConfigField.STATUS)
        self.children = [self.config_field]

class Summary(ShowType):
    'Displays status of a machine'
    def __init__(self):
        ShowType.__init__(self, "summary", "summary\tshows installation status summary")
        self.config_field = ConfigField.ConfigField(data_type=ConfigField.SUMMARY)
        self.children = [self.config_field]

class Dist(ShowType):
    'Displays basic machine configuration data from the server'
    def __init__(self):
        ShowType.__init__(self, "dist", "dist\tshow a python distribution tar-ball")
        self.config_field = ConfigField.ConfigField(data_type=ConfigField.DIST)
        self.children = [self.config_field]

class Machine(ShowType):
    'Displays basic machine configuration data from the server'
    def __init__(self):
        ShowType.__init__(self, "machine", "machine\tshow the configuration for one machine")
        self.config_field = ConfigField.ConfigField(data_type=ConfigField.MACHINE)
        self.children = [self.config_field]

class Include(ShowType):
    'Displays include information from the server'
    def __init__(self):
        ShowType.__init__(self, "include", "include\tshow a shared include file")
        self.config_field = ConfigField.ConfigField(data_type=ConfigField.INCLUDE)
        self.children = [self.config_field]

class Bom(ShowType):
    'Displays bill-of-materials ("bom") data from the server'
    def __init__(self):
        ShowType.__init__(self, "bom", "bom\tshow a bill of materials")
        self.config_field = ConfigField.ConfigField(data_type=ConfigField.BOM)
        self.children = [self.config_field]

class Package(ShowType):
    'Displays information about a package from the server'
    def __init__(self):
        ShowType.__init__(self, "package", "package\tdisplay information about a given package")
        self.config_field = ConfigField.ConfigField(data_type=ConfigField.PACKAGE)
        self.children = [self.config_field]

class User(ShowType):
    'Displays information about a user from the server'
    def __init__(self):
        ShowType.__init__(self, "user", "user\tdisplay information about a given user")
        self.config_field = ConfigField.ConfigField(data_type=ConfigField.USER)
        self.children = [self.config_field]

class History(PinshCmd.PinshCmd):
    'Displays command-line history (NOT from the server)'
    def __init__(self):
        PinshCmd.PinshCmd.__init__(self, "history")
        self.help_text = "history\tdisplay the history of commands"
        self.integer  = Integer.Integer(min_value=1, max_value=1000)
        self.children = [self.integer]
        self.cmd_owner = 1

    def cmd(self, tokens, _no_flag):
        "Shows the history for this user's bomsh session and before"
        if len(tokens) == 2 or tokens[-1].strip()=='':
            number = 20
        else:
            try:
                number = int(tokens[2])
            except ValueError:
                return FAIL, ["%s is not a number." % tokens[2]]
        hlen = readline.get_current_history_length()
        if hlen < number:
            number = hlen
        output = []
        for i in range(hlen-number, hlen):
            output.append("%4d\t%s" % (i, readline.get_history_item(i)))
        return OK, output

## FIXME: THIS IS BROKEN
#class Status(PinshCmd.PinshCmd):
#    '''Displays status information. SHOULD PULL FROM THE SERVER, BUT IS
#    BROKEN'''
#    def __init__(self):
#        PinshCmd.PinshCmd.__init__(self, "status")
#        self.help_text = "status\tstatus of a host"
#        #self.bom_host_field = BomHostField.BomHostField()
#        #self.children = [self.bom_host_field]
#        self.cmd_owner = 1
#
#    @classmethod
#    def printify(cls, input_objects):
#        'pretty-print status information'
#        text_list = list(input_objects)
#        text_list.sort()
#        output = []
#        if not text_list:
#            return []
#        max_length = max( [ len(t) for t in text_list ] )
#        columns = (system_state.termwidth - TERM_OVERCOUNT) / max_length
#        for i in range(0, len(text_list), columns):
#            newLine = ''
#            for item in text_list[i:i+columns]:
#                newLine += item.ljust(max_length+2)
#            output.append(newLine)
#        return output
#
#    def cmd(self, _tokens, _no_flag):
#        'prints status information, but does not work'
#        if no_flag:
#            return FAIL, []
#        if len(tokens) < 3:
#            return FAIL, ["Incomplete command."]
#        host_name = tokens[2]
#        status_file = os.path.join(system_state.server_home, "status", "%s.yml" % host_name)
#        if not os.path.isfile(status_file):
#            return FAIL, ["No status on file (%s)" % status_file]
#        #installed, broken = PackageField.getNamesFromProgress(host_name, False)
#        total_packages = PackageField.getPackageNamesFromBom(host_name)
#        missing = []
#        accounted_packages = list(installed.union(broken))
#        for item in total_packages:
#            found = False
#            for package_name in accounted_packages:
#                if package_name.startswith(item):
#                    found = True
#                    break
#            if not found:
#                missing.append(item)
#        if installed:
#            output = ["Installed:", [self.printify(list(installed))]]
#        else:
#            output = ["Installed:", [["NONE"]]]
#        if broken:
#            output += ["Broken:", [self.printify(list(broken))]]
#        else:
#            output += ["Broken:", [["NONE"]]]
#        if missing:
#            output += ["Not Installed:", [self.printify(list(missing))]]
#        else:
#            output += ["Not Installed:", [["NONE"]]]
#        return OK, output
        return OK, []


class Show(PinshCmd.PinshCmd):
#       bomsh# show machine localhost
#       [OK, ['default_user: root', 'ip_address: 127.0.0.1', 'packages:', '- TestPackageType4', 'platform: linux', 'test:', '  directory: /tmp/testthing', '  value: hello_kitty', '']]
#       bomsh# show summary localhost
#       [OK, ['broken: []', 'installed:', '- TestPackageType4-7', 'not_installed: []', 'status: 0', '']]
    '''bomsh# show bom qa
       [OK, ['- new_tomcat', '- new_apache', '- new_database', '']]
       bomsh# show dist test
       [OK, ['fields:', 'name: test', '']]
    '''
    def __init__(self):
        PinshCmd.PinshCmd.__init__(self, "show")
        self.help_text = "show\tdisplay components of the system"
        history = History()
        merged = Merged()
        machine = Machine()
        include = Include()
        status = Status()
        package = Package()
        user    = User()
        dist    = Dist()
        bom = Bom()
        summary = Summary()
        self.children = [merged, machine, include, bom, history, package, user, dist, status, summary]
        self.cmd_owner = 1




#!/usr/bin/env python

# BSD License
# Copyright (c) 2009, Peter Banka et al
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
# * Neither the name of the GE Security nor the names of its contributors may
#   be used to endorse or promote products derived from this software without
#   specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

'''This thing keeps track of the state of the bombardier shell. This module
is a design compomise because readline() requires it, really.'''


import readline, socket, sys, os
import yaml
from bombardier_core.static_data import OK, FAIL, USER, ADMIN, INFO, LOG_LEVEL_LOOKUP
from bombardier_core.static_data import LOG_LEVEL_LOOKUP

PERSONAL_CONFIG_FILE = "%s/.bdr_config" % os.environ.get("HOME")

NO_COLOR  = 'none'
DARK      = "dark"
LIGHT     = "light"


# possible modes
ENABLE = 0
C0 = 1
C1 = 2
C2 = 3
C3 = 4
F0 = 10
F1 = 11
F2 = 12
ALL = 99
FREE_TEXT = 100

class SystemState:
    '''A Singleton class that represents the state of the terminal'''

    class __impl:

        def __init__(self):
            self.logger = None
            self.batch = False
            self.state = []
            self.sub_prompt = None
            self.prompt = []
            self.globals = {}
            self.new_classes = []
            self.auth = USER
            self.full_prompt = ""
            self.username = None
            self.password = None
            self.comment_commands = []
            self.bom_connections = {}
            self.default_group = "root"
            self.debug = True
            self.global_config = {}
            self.personal_config = {}
            self.editor = "/usr/bin/vim"
            self.child_processes = []
            self.termwidth = 80
            self.termlen   = 23
            self.termcolor = LIGHT
            self.cnm_url = "http://127.0.0.1:8000"
            self.cnm_connector = None
            self.fp_out = sys.stdout
            self.fp_err = sys.stderr
            self.log_level = INFO
            self.ask_to_disconnect = False

    __instance = None

    def __init__(self):
        """ Create singleton instance """
        # Check whether we already have an instance
        if SystemState.__instance is None:
            # Create and remember instance
            SystemState.__instance = SystemState.__impl()

        # Store instance reference as the only member in the handle
        self.__dict__['_Singleton__instance'] = SystemState.__instance

    def __getattr__(self, attr):
        """ Delegate access to implementation """
        return getattr(self.__instance, attr)

    def __setattr__(self, attr, value):
        """ Delegate access to implementation """
        return setattr(self.__instance, attr, value)

    def set_auth(cls, auth_level):
        "Set the current level of authorization for the user"
        SystemState.__instance.auth = auth_level

    def add_personal_config_list(cls, option, value):
        '''The 'config_list' is the set of all configuration
        values in the user's .bomsh_config YAML file.'''
        if SystemState.__instance.personal_config.get(option):
            SystemState.__instance.personal_config[option].append(value)
        else:
            SystemState.__instance.personal_config[option] = [value]
        open(PERSONAL_CONFIG_FILE, 'w').write(yaml.dump(SystemState.__instance.personal_config))

    def load_config(cls):
        '''Loads configuration options from the user's .bomsh_config file'''
        if os.path.isfile(PERSONAL_CONFIG_FILE):
            config = yaml.load(open(PERSONAL_CONFIG_FILE).read())
            if config.get("ask_to_disconnect"):
                if type(config.get("ask_to_disconnect")) == type(True):
                    SystemState.__instance.ask_to_disconnect = \
                        config.get("ask_to_disconnect")
            if config.get("editor"):
                editor = config.get("editor")
                status = os.system("which %s" % editor)
                if status == OK:
                    SystemState.__instance.editor = editor
                else:
                    msg = "%% Editor %s cannot be found (using %s)."
                    print msg % (editor, self.editor)
            if config.get("username"):
                SystemState.__instance.username = config.get("username")
            if config.get("termwidth"):
                SystemState.__instance.termwidth = config.get("termwidth")
            termcolor = config.get("termcolor")
            if termcolor:
                if termcolor.upper().startswith("D"):
                    SystemState.__instance.termcolor = DARK
                elif termcolor.upper().startswith("L"):
                    SystemState.__instance.termcolor = LIGHT
                else:
                    SystemState.__instance.termcolor = NO_COLOR
            log_level = config.get("log_level", 'info').upper()
            if log_level in LOG_LEVEL_LOOKUP:
                SystemState.__instance.log_level = LOG_LEVEL_LOOKUP[log_level]
            cnm_url = config.get("cnm_url")
            if type(cnm_url) == type(''):
                SystemState.__instance.cnm_url = cnm_url
            debug = config.get("debug")
            if type(debug) == type(True):
                SystemState.__instance.debug = debug
            SystemState.__instance.editor = config.get("editor", "/usr/bin/vim")
            SystemState.__instance.personal_config = config

    def set_output(cls, fp_out):
        'set the output file handler'
        SystemState.__instance.fp_out = fp_out

    def set_err(cls, fp_err):
        'set the error file handler'
        SystemState.__instance.fp_err = fp_err

    def get_term_info(cls):
        '''tries to get infomration about the user's terminal'''
        try:
            var_list = os.environ["TERMCAP"].split(':')
            co = [ x for x in var_list if x.startswith("co") ][0]
            SystemState.__instance.termwidth = int(co.split("#")[-1])
            li = [ x for x in var_list if x.startswith("li") ][0]
            SystemState.__instance.termlen = int(li.split("#")[-1]) - 4
        except:
            SystemState.__instance.termlen = 23
            SystemState.__instance.termwidth = 80

    def set_prompt(cls):
        'used to set the current prompt'
        new_prompt = ''
        for state in SystemState.__instance.state:
            new_prompt += ('( %s )' % ' '.join(state))
        if not new_prompt:
            new_prompt =  "%s " % socket.gethostname()
            new_prompt += "(%s)" % (SystemState.__instance.username)
        new_prompt += "%s " % SystemState.__instance.prompt[-1]
        SystemState.__instance.full_prompt = new_prompt

    def push_prompt(cls, new_tokens):
        '''Used to enter a new sub-mode
        new_tokens -- completed tokens to be added to the prompt
        '''
        SystemState.__instance.state.append(new_tokens)
        cls.set_prompt()

    def get_prompt(cls):
        'used to find what the current prompt should look like'
        comment_string = ''
        if SystemState.__instance.comment_commands:
            comment_string = "(*)"
        return comment_string+SystemState.__instance.full_prompt

    def pop_prompt(cls):
        'used to exit a sub-mode'
        if len(SystemState.__instance.state) > 0:
            SystemState.__instance.state.pop()
            cls.set_prompt()
            return OK
        else:
            return FAIL

    def get_state_tokens(cls, tokens):
        'this will return a modified list of tokens based on the current state'
        new_tokens = []
        for token_list in SystemState.__instance.state:
            new_tokens.extend(token_list)
        if new_tokens and tokens == []:
            tokens = ['']
        return new_tokens + tokens

    def reprompt(cls):
        'This is used under some weird conditions that I forget'
        print >> sys.stderr
        print >> sys.stderr, cls.get_prompt() + readline.get_line_buffer(),

    def current_state(cls):
        return SystemState.__instance.state[-1]




#!/usr/bin/python
# BSD License
# Copyright (c) 2009, Peter Banka et al
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
# * Neither the name of the GE Security nor the names of its contributors may
#   be used to endorse or promote products derived from this software without
#   specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

'''A [PinshCmd] object that provides a field with automated completion.
Supports access to 'configurable items' on the bombardier web server'''

import PinshCmd
from bombardier_core.static_data import OK, FAIL, PARTIAL, COMPLETE, NO_MATCH
from Exceptions import UnexpectedDataException
from SystemStateSingleton import SystemState
system_state = SystemState()

MERGED = 1
MACHINE = 2
INCLUDE = 3
BOM = 4
PACKAGE = 5
USER = 6
DIST = 7
STATUS = 8
SUMMARY = 9

DEFAULT_MACHINE = """ip_address: 127.0.0.1
default_user: root
platform: rhel5
"""

DEFAULT_PACKAGE = """class_name: Module.Class
dependencies: []
injectors: {}
libs: {}
package-version: 5
release: 0
"""

DEFAULT_BOM = """
-
"""

class ConfigField(PinshCmd.PinshCmd):
    '''The server keeps track of several types of configuration data.
    This class can provide command-line completion for those data objects'''
    def __init__(self, name = "configField", data_type=MERGED, strict=True, new=False):
        '''
        name -- Not used except in debugging
        data_type -- which type of configuration data is this object supposed
                     to match? Can be MERGED, MACHINE, INCLUDE, BOM, DIST, 
                     or PACKAGE
        strict -- The preferred_names() method can be either liberal such that
                  'local' will match 'localhost' or can be strict such that
                  'local' will not match 'localhost'
        '''
        PinshCmd.PinshCmd.__init__(self, name)
        self.help_text = "<configurationField>\t"\
                         "a dot-delimeted configuration value"
        self.data_type = data_type
        self.cmd_owner = 0
        self.strict = strict # take only exact matches
        if data_type == MACHINE:
            self.directory = "machine"
        elif data_type == MERGED:
            self.directory = "merged"
        elif data_type == INCLUDE:
            self.directory = "include"
        elif data_type == BOM:
            self.directory = "bom"
        elif data_type == PACKAGE:
            self.directory = "package"
        elif data_type == USER:
            self.directory = "user"
        elif data_type == DIST:
            self.directory = "dist"
        elif data_type == STATUS:
            self.directory = "status"
        elif data_type == SUMMARY:
            self.directory = "summary"
        self.new = new

    def get_object_list(self):
        'returns a list of all self.data_type things'
        url = "json/%s/search/" % self.directory
        data = system_state.cnm_connector.service_yaml_request(url)
        #print "get_object_list: url: %s\ndata:%s\n" %(url, data)
        if self.directory == "user":
            object_list = [ x.get("fields").get("username") for x in data ]
        else:
            object_list = [ x.get("fields").get("name") for x in data ]
        return object_list

    def get_data(self, first_token_name):
        'returns a list of all configuration objects of this type'
        url = "json/%s/name/%s" % (self.directory, first_token_name)
        data = system_state.cnm_connector.service_yaml_request(url)
        return data

    def get_top_level_data(self, tokens, index):
        '''someone typed in something like 'show machine localho'. Our job is
        to figure out that localhost is the object that needs to be found and
        to return the dictionary for that object.
        '''
        #print "my directory: %s" % self.directory
        #print "get_top_lvl_data: tokens", tokens
        partial_first = tokens[index]
        object_names = self.get_object_list()
        #print "OBJECT NAMES:",object_names
        first_token_names = []
        for ftn in object_names:
            if ftn.lower().startswith(partial_first.lower()):
                #print "%s starts with the same as partial_first: %s" % (ftn, partial_first)
                first_token_names.append(ftn)
        if len(first_token_names) == 0:
            return [], {}
        if tokens[index] in first_token_names:
            first_token_names = [tokens[index]]
        if len(first_token_names) > 1:
            return first_token_names, {}
        first_token_name = first_token_names[0]
        data = self.get_data(first_token_name)
        return [first_token_name], data

    def get_specific_data(self, tokens, index):
        '''used with the show command to display data to the screen'''
        tokens[index] = tokens[index].replace('"', '')
        try:
            first_token_names, data = self.get_top_level_data(tokens, index)
        except TypeError:
            return FAIL, "Unable to read data from server"
        if len(first_token_names) != 1:
            return '' # EXPERIMENTAL CHANGE from []
        return data

    def post_specific_data(self, tokens, index, new_data):
        '''used with the edit command to upload data to server'''
        tokens[index] = tokens[index].replace('"', '')
        if not self.new:
            try:
                first_token_names, data = self.get_top_level_data(tokens, index)
            except TypeError:
                return FAIL, "Unable to read data from server"
            if len(first_token_names) < 1:
                msg = "%s was not found on the server to upload to" % tokens[index]
                raise UnexpectedDataException(msg)
            url = "json/%s/name/%s" % ( self.directory, first_token_names[0])
        else:
            url = "json/%s/name/%s" % ( self.directory, tokens[index])
        post_data = {"yaml": new_data}
        output_dict = system_state.cnm_connector.service_yaml_request(url,
                                                               post_data=post_data)
        return output_dict["command_status"], output_dict["command_output"]

    def preferred_names(self, tokens, index):
        '''Provide a list of names that the system would prefer to use, other
        than that which was typed in by the user. For example, 'sho mach localh'
        will return 'localhost' for the machine name if strict is off,
        otherwise, it will return 'localh'.
        '''
        #print "PN: begin self.string = ", self.strict
        tokens[index] = tokens[index].replace('"', '')
        if not self.strict:
            return tokens[index:]
        try:
            first_token_names, data = self.get_top_level_data(tokens, index)
        except TypeError:
            return FAIL, "Unable to read data from server"

        #print "PN: first_token_names: ", first_token_names
        if len(first_token_names) == 0:
            if self.new:
                return tokens[index:]
            return []
        if self.new:
            if not tokens[index] in first_token_names:
                return tokens[index:]
            else:
                return []
        if len(first_token_names) > 1:
            return first_token_names
        first_token_name = first_token_names[0]
        return [first_token_name]

    def match(self, tokens, index):
        '''Determines if what has been typed in by the user matches a
        configuration item that the system is keeping track of.'''
        possible_matches = self.acceptable_names(tokens, index)
        if not possible_matches:
            return NO_MATCH, 1
        if len(possible_matches) > 1:
            return PARTIAL, 1
        return COMPLETE, 1

    def get_default_data(self):
        if self.data_type == MACHINE:
            return DEFAULT_MACHINE
        elif self.data_type == BOM:
            return DEFAULT_BOM
        elif self.data_type == PACKAGE:
            return DEFAULT_PACKAGE
        return ""



#!/usr/bin/python
# BSD License
# Copyright (c) 2009, Peter Banka et al
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
# * Neither the name of the GE Security nor the names of its contributors may
#   be used to endorse or promote products derived from this software without
#   specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

"""This provides a top-level command for the bomsh shell. This is for
managing machines in the network, and subclasses the [PinshCmd] module
"""

__author__ =  'Peter Banka'
__version__ = '1.0'

import PinshCmd
from ConfigField import ConfigField, MACHINE, DIST, PACKAGE
from bombardier_core.static_data import OK, FAIL
from SystemStateSingleton import SystemState, ENABLE
from PackageField import PackageField, FIX, PURGE, NOT_INSTALLED, INSTALLED
import PackageActionField
from Exceptions import MachineTraceback, CommandError
from JobNameField import JobNameField
import Integer
from Ssh import Ssh
from Show import Status, Summary
import Edit
system_state = SystemState()
import libUi, time, yaml

URL_LOOKUP = {'test': "json/machine/start_test/%s",
              'dist': "json/machine/dist/%s",
              'init': "json/machine/init/%s",
              'reconcile': "json/machine/reconcile/%s",
              'check-status': "json/machine/check_status/%s",
              'status': "json/machine/status/%s",
              'enable': "json/machine/enable/%s",
              'disable': "json/machine/disable/%s",
              'setup': "json/machine/setup/%s",
             }

def setup_test():
    fresh_status_yaml = """ 
client_version: 1.00-634
core_version: 1.00-629
install-progress: {}
timestamp: 1251918530.6349609"""

    status_file = "/opt/spkg/localhost/status.yml"
    open(status_file, "w").write(fresh_status_yaml)

class Machine(PinshCmd.PinshCmd):
#    '''bomsh# package Testpackagetype4 install localhost 
##       [OK, ['install OK: TestPackageType4-7', 'verify OK: TestPackageType4-7']]
#    '''
#       bomsh# package Testpackagetype4 install localhost 
#       [OK, ['install OK: TestPackageType4-7', 'verify OK: TestPackageType4-7', 'Finished install for TestPackageType4.']]
#       bomsh# package TestPackagetype4 echo localhost 
#       [OK, {'test file exists': 'True', 'contents of test file': 'The Quick Brown Fox Jumped over the Lazy Dog.'}]
#    '''
#       bomsh# package Testpackagetype4 uninstall localhost 
#       [OK, ['uninstall OK: TestPackageType4-7']]
#       bomsh# package bogusPackage install localhost 
#       [FAIL, []]
#    '''
#       bomsh# machine localhost enable
#       [OK, []]
#       bomsh# machine localhost test
#       [OK, ['Machine localhost is ready to take commands.']]
#       bomsh# machine localhost dist test
#       [OK, ['localhost updated with test']]
#       bomsh# machine localhost init
#       [OK, []]
    '''
       bomsh# machine localhost reconcile
       [OK, ['install OK: TestPackageType4-7', 'verify OK: TestPackageType4-7', 'Finished installing']]
       bomsh# machine localhost check-status
       [OK, []]
       bomsh# machine localhost status purge TestPackageType4-7
       [OK, ['TestPackageType4-7 has been removed from localhost status']]
       bomsh# machine localhost status fix TestPackageType4
       [OK, ['TestPackageType4-7 has been set to INSTALLED.']]
       bomsh# machine localhost status purge TestPackageType4-7
       [OK, ['TestPackageType4-7 has been removed from localhost status']]
    '''
#       bomsh# machine localhost disable
#       [OK, []]
#       bomsh# machine localhost enable
#       [OK, []]

    def __init__(self):
        """Top-level object has a 'test' child: test the machine
        """
        PinshCmd.PinshCmd.__init__(self, "machine")
        self.help_text = "machine\tcommands that operate on a given machine"
        self.cmd_owner = 1
        self.machine_field = None
        self.children = []
        self.build_children()
        self.auth = ENABLE

    def build_children(self):
        # Top-level commands
        self.machine_field = ConfigField(data_type=MACHINE)
        self.children = [self.machine_field]

        # Second-level commands
        test = PinshCmd.PinshCmd("test", "test\ttest a machine connection")
        dist = PinshCmd.PinshCmd("dist", "dist\tdeploy a library to a machine")
        enable = PinshCmd.PinshCmd("enable", "enable\tshare ssh keys with a machine")
        disable = PinshCmd.PinshCmd("disable", "disable\tremove ssh key from a machine")
        init = PinshCmd.PinshCmd("init", "init\tintialize bombardier on a machine")
        reconcile = PinshCmd.PinshCmd("reconcile", "reconcile\treconcile machine state to bill of materials")
        check_status = PinshCmd.PinshCmd("check-status", "check-status\treview the current state of the system")
        status = PinshCmd.PinshCmd("status", "status\tstatus reporting or manipulation")
        summary = PinshCmd.PinshCmd("summary", "summary\tshow the digested status information")
        install = PinshCmd.PinshCmd("install", "install\tinstall a package")
        uninstall = PinshCmd.PinshCmd("uninstall", "uninstall\tremove a package")
        configure = PinshCmd.PinshCmd("configure", "configure\tconfigure a package")
        verify = PinshCmd.PinshCmd("verify", "verify\tverify a package")
        execute = PinshCmd.PinshCmd("execute", "execute\trun an action on a package")
        ssh = PinshCmd.PinshCmd("ssh", "ssh\tconnect directly to this machine from the cli")
        edit = PinshCmd.PinshCmd("edit", "edit\tedit the configuration for this machine")
        push = PinshCmd.PinshCmd("push", "push\tsend cleartext configuration data to machine for troublehsooting")
        unpush = PinshCmd.PinshCmd("unpush", "unpush\tremove cleartext configuration data from machine")
        setup = PinshCmd.PinshCmd("setup", "setup\tinstall bombardier core and client onto a machine with depedencies")
        job = PinshCmd.PinshCmd("job", "job\toperations on the job queues")
        self.machine_field.children = [test, dist, init, enable, ssh, summary,
                                       disable, check_status, status, edit,
                                       reconcile, execute, install, uninstall,
                                       configure, verify, push, unpush, setup,
                                       job]

        # Third-level commands
        dist_field = ConfigField(data_type=DIST)
        dist.children = [dist_field]
        fix = PinshCmd.PinshCmd("fix", "fix\tsets the state of a package to INSTALLED")
        purge = PinshCmd.PinshCmd("purge", "purge\tremoves a package from the status of a machine")
        status.children = [fix, purge]
        installed_package_field = PackageField(INSTALLED)
        not_installed_package_field = PackageField(NOT_INSTALLED)
        executable_package_field = PackageField(INSTALLED)
        install.children = [not_installed_package_field]
        uninstall.children = [installed_package_field]
        verify.children = [installed_package_field]
        configure.children = [installed_package_field]
        execute.children = [executable_package_field]

        clear_broken = PinshCmd.PinshCmd("clear-broken", "clear-broken-jobs\tclear the broken jobs")
        stop = PinshCmd.PinshCmd("stop", "stop\timmediately kill all active and pending jobs")
        show = PinshCmd.PinshCmd("show", "show\tshow active, pending, broken, and finished jobs")
        view = PinshCmd.PinshCmd("view", "view\tview a running, finished, or broken job")
        job.children = [clear_broken, stop, show, view]

        # Fourth-level commands
        fix.children = [PackageField(action_type=FIX)]
        purge.children = [PackageField(action_type=PURGE)]
        package_actions = PackageActionField.PackageActionField()
        executable_package_field.children = [package_actions]
        job_name_field = JobNameField()
        view.children = [job_name_field]

    def analyze_output(self, machine_name, tokens, output, post_data):
        command = tokens[2].lower()
        if command == "test":
            for i in range(1,5):
                if "Testing %d/4" % i not in output["complete_log"]:
                    return FAIL, "Invalid output from server"
                return OK, [ "Machine %s is ready to take commands." % machine_name ]
        elif command == "dist":
            if output["command_status"] == OK:
                return OK, ["%s updated with %s" % (machine_name, post_data["dist"])]
            else:
                return FAIL, ["%s FAILED to install %s" % (machine_name, post_data["dist"])]
        return output["command_status"], output["command_output"]

    def check_machine_name(self, tokens, no_flag):
        possible_machine_names = self.machine_field.preferred_names(tokens, 1)
        machine_name = possible_machine_names[0]
        if no_flag:
            raise CommandError("NO cannot be used here")
        if len(possible_machine_names) == 0:
            raise CommandError("Unknown machine name: %s" % tokens[2])
        if len(possible_machine_names) > 1:
            raise CommandError("Ambiguous machine name: %s" % tokens[2])
        return machine_name

    def get_url_and_post(self, machine_name, tokens):
        command = tokens[2].lower()
        if command not in URL_LOOKUP:
            raise CommandError("%s is not a valid command" % command)

        post_data = {}
        url = URL_LOOKUP[command] % machine_name
        if command == "dist":
            post_data = {"dist": tokens[-1]}
        elif command in [ "enable", "setup" ]:
            prompt = "%s administrative ssh password: " % machine_name
            password = libUi.pwd_input(prompt)
            yml_dict = yaml.dump( {"password" : password } )
            post_data = {"yaml" : yml_dict}
        elif command == "status":
            if len(tokens) > 3:
                if len(tokens) < 5:
                    raise CommandError("Incomplete command")
                sub_command = tokens[3].lower()
                if sub_command not in ["fix", "purge"]:
                    raise CommandError("%s is not a valid command" % sub_command)
                post_data["action"] = sub_command
                post_data["machine"] = machine_name
                url = "/json/package_action/%s" % tokens[4]
        return url, post_data

    def cmd(self, tokens, no_flag):
        """
        tokens -- all of the keywords passed in the command string, parsed
        no_flag -- whether the 'no' keyword was used in the command string
        """
        if len(tokens) < 2:
            raise CommandError("Incomplete command.")
        machine_name = self.check_machine_name(tokens, no_flag)
        if len(tokens) == 2:
            system_state.push_prompt(["machine", machine_name])
            return OK,[]

        command = tokens[2].lower()

        if command == "edit":
            return Edit.Machine().cmd(["edit", "machine", machine_name], 0)

        if command == "ssh":
            return Ssh().cmd(["ssh", machine_name], 0)

        if command == "status" and len(tokens) == 3:
            return Status().cmd(["show", "status", machine_name], 0)

        if command == "summary" and len(tokens) == 3:
            return Summary().cmd(["show", "summary", machine_name], 0)

        if command == "push":
            url = "/json/machine/push/%s" % machine_name
            post_data = {"machine": machine_name}
            job_name = system_state.cnm_connector.get_job(url, post_data)
            return system_state.cnm_connector.watch_jobs([job_name])

        if command == "unpush":
            url = "/json/machine/unpush/%s" % machine_name
            post_data = {"machine": machine_name}
            job_name = system_state.cnm_connector.get_job(url, post_data)
            return system_state.cnm_connector.watch_jobs([job_name])

        if command == "job":
            sub_command = "show"
            if len(tokens) >= 4:
                sub_command = tokens[3]
            
            post_data = None
            if sub_command == "stop":
                url = "/json/machine/stop-jobs/"
                post_data = {"machine": machine_name}
            elif sub_command == "clear-broken":
                url = "/json/machine/clear-broken-jobs/"
                post_data = {"machine": machine_name}
            elif sub_command == "view":
                job_name = tokens[4]
                return system_state.cnm_connector.watch_jobs([job_name])
            else:
                url = "/json/machine/show-jobs/%s" % machine_name
                output = system_state.cnm_connector.service_yaml_request(url)
                return output["command_status"], output
            output = system_state.cnm_connector.service_yaml_request(url, post_data=post_data)
            return output["command_status"], output["command_output"]

        if command in ["install", "uninstall", "verify",
                       "configure", "execute"]:
            if command == "execute":
                if len(tokens) != 5:
                    raise CommandError("Incomplete Command")
                command = tokens[4]
            if len(tokens) < 3:
                raise CommandError("Incomplete command.")
            package_name = tokens[3]
            machine_name = tokens[1]

            url = "/json/package_action/%s" % package_name
            post_data = {"machine": machine_name,
                         "action": command}
        else:
            url, post_data = self.get_url_and_post(machine_name, tokens)

        try:
            job_name = system_state.cnm_connector.get_job(url, post_data)
            output = system_state.cnm_connector.watch_jobs([job_name])
            return self.analyze_output(machine_name, tokens, output[job_name], post_data)
        except MachineTraceback, m_err:
            libUi.process_traceback(m_err)
            return FAIL,[]




#!/usr/bin/python
import bombardier_core.Cipher
import yaml
import sys
import getpass

filename = sys.argv[1]
bombardier_core.Cipher.VALID_CHARS.append('\n')

def enc_dict(dictionary, cipher):
    for key in dictionary:
        if key.startswith('enc'):
            continue
        value = dictionary[key]
        if type(value) == type({}):
            dictionary[key] = enc_dict(dictionary[key], password)
        elif type(value) == type("string"):
            cipher_text = cipher.encrypt(value, password)
            del dictionary[key]
            dictionary['enc_%s' % key] = cipher_text
    return dictionary

password = getpass.getpass("need the password to use:")
cipher = bombardier_core.Cipher.Cipher(password)
data_dict = yaml.load(open(filename).read())
data_dict = enc_dict(data_dict, cipher)
enc_filename = "%s.enc" % filename
open(enc_filename, 'w').write(yaml.dump(data_dict))

a = yaml.load(open(enc_filename).read())
libCipher.decryptLoop(a, password)
print "Round-trip data:", a




#!/usr/bin/python
# BSD License
# Copyright (c) 2009, Peter Banka et al
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
# * Neither the name of the GE Security nor the names of its contributors may
#   be used to endorse or promote products derived from this software without
#   specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

'''A [PinshCmd] object that provides a field with automated completion.
Will be a basic package action or one that the package exposes'''


import PinshCmd
from bombardier_core.static_data import OK, FAIL, PARTIAL, COMPLETE, NO_MATCH
from SystemStateSingleton import SystemState
system_state = SystemState()

class PackageActionField(PinshCmd.PinshCmd):
    '''This class can provide command-line completion for actions if it
    exposes methods'''
    def __init__(self, name = "packageActionField"):
        'Just sets up the basics'
        PinshCmd.PinshCmd.__init__(self, name, token_delimeter = '')
        self.help_text = "<packageActionField>\t"\
                         "an action that can be taken on a package"
        self.cmd_owner = 0

    def get_package_name(self, package_name):
        'returns a list of all packages that might match'
        url = "json/package/search/%s" % package_name
        data = system_state.cnm_connector.service_yaml_request(url)
        package_list = [ x.get("fields").get("name") for x in data ]
        return package_list

    def get_package_data(self, package_name):
        'returns a list of all configuration objects of this type'
        url = "json/package/name/%s" % (package_name)
        data = system_state.cnm_connector.service_yaml_request(url)
        return data

    def preferred_names(self, tokens, index):
        '''Provide a list of names that the system would prefer to use, other
        than that which was typed in by the user. For example, 'sho mach localh'
        will return 'localhost' for the machine name if strict is off,
        otherwise, it will return 'localh'.

        '''
        package_name = tokens[index-1]
        package_data = self.get_package_data(package_name)
        actions = package_data.get("executables")
        possible_actions = []
        for action in actions:
            if action.lower().startswith(tokens[index].lower()):
                possible_actions.append(action)
        #print "possible actions:",possible_actions
        return possible_actions

    def match(self, tokens, index):
        '''Determines if what has been typed in by the user matches a
        possible package action'''
        possible_matches = self.acceptable_names(tokens, index)
        if not possible_matches:
            return NO_MATCH, 1
        if len(possible_matches) > 1:
            return PARTIAL, 1
        return COMPLETE, 1
